package cia_components.ppc;

import java.util.*;

public class Slot
{
	Sentence parent_sentence;
	int slot_number;
	
	static int slot_counter = -1;
	// The initial value of this variable arises from the need to ignore slot no. 0 
	// which contains the header line transmitted by CDG.
	
	Vector <String> homonym_lines;
	Vector <Word> slot_word_objects;
	String slot_string;
	int slot_size;
	Vector <String> homonym_descriptions;
	
	public Slot( Sentence parent, Vector <String> block )
	{	
		set_Parent_Sentence( parent );
		slot_counter ++;
		set_Slot_Number( slot_counter );
		// System.out.println( "Making slot no. " + slot_number );
		
		homonym_lines = block;

		set_Slot_Size( homonym_lines.size() );
		// System.out.println( "Making slot with input block size: " + get_Slot_Size() );
								
		// Then create a new Word object for each homonym.
		make_Slot_Words();
	}
		
	
	private void make_Slot_Words()
	{	
		/*
			From the CDG input lines this method creates the individual Word objects for this slot.
			The slot positions are filled with Word objects according to the sequence in which their 
			lexicon entries have been retrieved from CDG2. The first word object takes position 1.
		*/
		
		slot_word_objects = new Vector <Word> ();
		
		homonym_descriptions = new Vector <String> ();

		int position = 0;
		
		Word word; 
		
		for ( String line : homonym_lines )
		{
			// System.out.println( "Line received for conversion into word object: " + line );

			position ++;

			if ( get_Slot_Number() > 0 )
			{
				// System.out.println( "Slot number : " + get_Slot_Number() );
				
				word = new Word( this, position, line );
				// Note that the entire line, i. e.: the entire lexicon entry, is handed over to the Word object constructor for tokenisation.
				
				slot_word_objects.add( word );
				// The newly generated Word object is added to the slot's vector of words.
				
				// System.out.println( "This word has position no. " + word.get_Position_In_Slot() + " in slot no. " + get_Slot_Number() );

				add_to_Parent_Sentence( word );
			}
		}
	}
	

	/** This method adds a word object to the vector of this slot's 
	 * parent sentence of all word objects.
	 */

	private void add_to_Parent_Sentence( Word word )
	{
		get_Parent_Sentence().add_Sentence_Word( word );
	}
	
	
	public String get_Slot_String()
	{
		return slot_string;
	}
	    
	    
	public Vector <Word> get_All_Word_Objects()
	{
		return slot_word_objects;
	}
	
    
	public Word get_Word_Object( int slot_position )
	{
		Word result = (Word) slot_word_objects.elementAt( slot_position -1 );
		// The correction of -1 is to adjust the position in a slot to the vector index which starts at 0 rather than 1.
		return result;
	}
		
			
	public String get_Input_Line_From_CDG_Main( int i )
	{
		/*
			The correction '-1' has been introduced to account for the fact that the vector is being filled
			from index '0' while references to elements should be made in line with natural language expectations
			('the first element of ...').
		*/
		return (String) homonym_lines.elementAt( i -1 );
	}

	
	public int get_Slot_Number()
	{
		return slot_number;
	}
	
	
	public int size()
	{
		return slot_size;
	}
	
	
	public void set_Slot_Size( int input_size )
	{
		slot_size = input_size;
		// System.out.println( "Slot size: " + slot_size );
	}
	
	
	public void set_Slot_Number( int input_number )
	{
		slot_number = input_number;
		// System.out.println( "Slot number: " + slot_number );
	}


	public void set_Slot_String( String input_word )
	{
		slot_string = input_word;
		// System.out.println( "Slot string for slot no. " + slot_number + " :\t" + slot_string );
	}


	private void set_Parent_Sentence( Sentence input_sentence )
	{
		parent_sentence = input_sentence;
	}
	
	
	public Sentence get_Parent_Sentence()
	{
		return parent_sentence;
	}

	
	public void add_Description( String description )
	{
		homonym_descriptions.add( description );
		
		// Also pass on this description to the parent sentence.
		this.get_Parent_Sentence().add_Description( description );
	}
	
	
	public Vector <String> get_Descriptions()
	{
		return homonym_descriptions;
	}

	
	public Word get_Last_Word()
	{
		return slot_word_objects.lastElement();
	}
}