#! /usr/bin/perl -w

###	This script compares the sizes of the lexicon and its generation input files 
###	for WCDG1 and WCDG2. Variants of listed words are not considered separate 
###	lexical entries and hence are ignored.

use strict;
use locale;

my %sizes;

# List of files to process.

my @files = ( "Verben.txt", "Nomen.txt", "Namen.txt", "Adjektive.txt", "deutsch-lexikon.cdg" );

# List of file-specific regexes.

my @regexes = ( qr 'verb [^ ]+\s+', qr 'noun \w+', qr '(Firma|Nachname|Produ|Region|Vorname) [^ ]+', qr 'adj [^ ]+', qr ':=' );


for my $file ( @files )
{
	my $regex = shift @regexes;
	
	print $file, "\n";
	
	for my $version ( "", 2 )
	{
		my $path = "/home/mccrae/cdg".$version."/grammar/deutsch/";
		
		{			
			open FILE, "<", $path.$file or die "Could not open $file. $!";
			
			while ( <FILE> )
			{
				next if /^$/;
				$sizes{ $version }{ $file } ++ if /$regex/;
			}
		}
		printf "CDG%-3s %d entries.\n", $version.":", $sizes{ $version }{ $file };
	}
	
	# Use the length of the longer number of entries for print alignment.
	
	my $len;
	 
	if ( $sizes{ 2 }{ $file } >= $sizes{ "" }{ $file } )
	{
		$len = length $sizes{ 2 }{ $file };
	}
	else 
	{
		$len = length $sizes{ "" }{ $file };
	}
	
	printf "Delta: %${len}d entries.\n", ( $sizes{ 2 }{ $file } - $sizes{ "" }{ $file } );
	printf "\% Change: %.2f%.\n\n", ( $sizes{ 2 }{ $file } / $sizes{ "" }{ $file } -1 )*100;
}