/* 
 * Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
 *  
 * This file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. For
 * more details read COPYING in the root of this distribution.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

/*
 * File: functions.h
 * Author: Ingo Schroeder
 * Purpose: header file for functions.c
 * Birth: 17/3/97
 *
 */

#ifndef FUNCTIONS_H
#define FUNCTIONS_H

/* -- INCLUDES ---------------------------------------------------------- */
#include "cdg.h"
#include <blah.h>
#include "constraintnet.h"
#include "lexemgraph.h"
#include "eval.h"

/* -- MACROS ------------------------------------------------------------ */
/* -- TYPE DEFINITIONS -------------------------------------------------- */
typedef struct {
   String functor;
   VTFunction *function;
   Boolean is_context_sensitive;
   String arity;
   String functionName;
   String doc;
} CDGFunctionStruct;
typedef CDGFunctionStruct* CDGFunction;

/* -- FUNCTIONS --------------------------------------------------------- */
extern CDGFunction funcGetFunction(String functor);
extern String funcGetFunctionName(String functor);

/*
 * These functions should not be exported! Fix the compiler.
 */
extern Value funcAbs(List args, Value v, LexemGraph lg, Vector context);
extern Value funcAcoustics(List args, Value v, LexemGraph lg, Vector context);
extern Value funcCat(List args, Value v, LexemGraph lg, Vector context);
extern Value funcDistance(List args, Value v, LexemGraph lg, Vector context);
extern Value funcExp(List args, Value v, LexemGraph lg, Vector context);
extern Value funcLookup(List args, Value v, LexemGraph lg, Vector context);
extern Value funcMatch(List args, Value v, LexemGraph lg, Vector context);
extern Value funcMax(List args, Value v, LexemGraph lg, Vector context);
extern Value funcMin(List args, Value v, LexemGraph lg, Vector context);
extern Value funcPredict(List args, Value v, LexemGraph lg, Vector context);
extern Value funcParens(List args, Value v, LexemGraph lg, Vector context);
extern Value funcQuotes(List args, Value v, LexemGraph lg, Vector context);
extern Value funcPhrasequotes(List args, Value v, LexemGraph lg, Vector context);
extern Value funcParent(List args, Value v, LexemGraph lg, Vector context);
extern Value funcHeight(List args, Value v, LexemGraph lg, Vector context);
extern int funcHeightImpl(LexemNode ln, int levelno, LexemGraph lg, Vector context);
extern int funcParentImpl(LexemGraph lg, Vector context, int a, int levelno, String label);
extern Value funcVisual(List args, Value v, LexemGraph lg, Vector context);
/* -- VARIABLES --------------------------------------------------------- */
extern CDGFunctionStruct eval_functions[];

/* ---------------------------------------------------------------------- */
#endif /* don't insert anything after this #endif */
