/*
 * Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
 *
 * This file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. For
 * more details read COPYING in the root of this distribution.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Birth: 1.9.98
 *
 */

/* -- INCLUDES ---------------------------------------------------------- */
#include <config.h>

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <blah.h>
#include "cdg.h"
#include "command.h"
#include "input.h"
#include "eval.h"
#include "statistics.h"
#include "levelvalue.h"
#include "constraintnet.h"
#include "lexemgraph.h"
#include "netsearch.h"
#include "increment.h"
#include "timer.h"
#include "hook.h"
#include "parse.h"
#include "set.h"
#include "transform.h"
#include "frobbing.h"
#include "write.h"
#include "virtualnode.h"
#include "tagger.h"


/* -- EXTERNAL VARIABLES ------------------------------------------------ */
IncrementalStep incrMostRecentlyCreatedIncrementalStep = NULL;

Boolean incrResetUnremovableConflicts = TRUE; // if the list of unremovable conflicts should be cleared after every step
Number incrPressureRedux = 0.0; // the factor the global minimum (pressure) will be reduced by between steps

FreezingLevelType incrFreezeLevel = FLTnone; // setting for the freezing heuristics: on, off or simulated
FreezingHeuristic incrFreezingHeuristics = FHconflictFreeProjective;
Boolean incrFreezeIncoming = TRUE;  // besides the edges inside the domain, edges having the domains slot as regent will be frozen, too
Boolean incrOptimizeAnalysis= FALSE; // force a try to find the best attachment for the new domain before frobbing kicks in
Boolean incrScoreFluctuation= TRUE; // recalculate certain constraints that might fluctuate
Boolean incrEdgeReactivation= FALSE;// check if edges with score zero have changed their score and need to be reactivated
									// rather time consuming and not necessary with tested tagger-grammar combination
Boolean incrRetryUnfrozen = FALSE;  // unfreeze and retry if score is to low
Boolean incrOnlyFreezeWhenGoalMet= TRUE; // don't try to freeze if last steps goal was not met
Boolean incrAutoFinalize= FALSE;     // says, if the last token should automatically be treated as the end of the sentence
Boolean incrUseFrontierPredicate = TRUE; 	 // activates the frontier predicate, otherwise the predicate always returns false,
									 //  the predicate returns true, if the given word is one of the last n in a partial input
float incrUnfreezeThreshold= 0.5;    // if score is lowered by this factor, try again unfrozen
float incrFrobbingGoalDecline = 0.0; // frobbing goal will be reduced by this factor if goal was not met, set to 0.0 to disable

Boolean incrFreezeHeads    = FALSE; // if true the heads of a substructure will be frozen, too, used for radical freezing
int     incrLookaheadSize  = 3;     // lookahead size of lookahead freezing
int 	incrTaggerLookaheadSize = 0; // number of extra tokens to be handed to the tagger

String labelExceptionProperty = NULL; // property name for the list of Labels that have to be spared from freezing, as they might appear in a nonprojective way
String lexemeAttributeExceptionProperty = NULL; // property name for list of attributes, frozen subtrees might need to stay flexible regarding certain attributes like 'case'
List incrVolatileConstraints = NULL; // list of conflicts that will probably disappear again
List incrPersistentConstraints = NULL; // list of conflicts that will never disappear with new input
//List incrFluctuatingConstrSections = NULL; // list of constraint sections that might change score in an incremental environment
//List incrFluctuatingConstrNonspec = NULL; // list of constraints that might change score for nonspec- edges in an incremental environment
List incrTemporarilyAllowedConstraints = NULL; // list of constraints that act hard    (penalty = 0.0) in complete sentences
List incrTempPenalizedConstraints = NULL; // list of constraints that don't apply (penalty = 1.0) in complete sentences

Boolean incrTaggerFeedback  = FALSE; // should feedback be provided to the tagger?
int incrTaggerFeedbackDelay = 0; // how many tokens should the feedback of the tagger be delayed, i.e., how many tokens at the right border will not be fed back to the tagger
Boolean incrTaggerFeedbackRetag =TRUE; // should the tagger retag all tags after a changed tag?

Boolean alwaysShowAllSolutions = FALSE; // if set, it will override suppressPartialSolutions

Boolean initVNsUnused = TRUE; // if set, initial analyses only use nil regents and empty labels for virtual nodes

// development related settings, it's not recommended to change these, as the programm might not work properly
Boolean incrResetLimits = TRUE; // limits of LVs and Lexemes will be reseted between incremental steps
Boolean incrOptimizeNet = FALSE; //execute optimizenet once after extending before frobbing, optimize net is usually called after successful repair attempt



/* -- CONSTANTS ----------------------------------------------------------*/
//String CV_NONSPEC_ID_SYN = "NONSPEC-Kante (SYN)";
//String CV_NONSPEC_ID_ARG = "NONSPEC-Kante (ARG)";
String cdgfolder = NULL;

/********** Constructors *****************/


/*
 * registers incremental specific set variables
 */
void incrInitialize() {
	cdgfolder = getenv("CDGDIR")?strCat(getenv("CDGDIR"),"/"):"";

	setRegister("resetUnremovableConflicts", SET_BOOL, &incrResetUnremovableConflicts, NULL, NULL, NULL, NULL);
	//setRegister("resetLimits", SET_BOOL, &incrResetLimits, NULL, NULL, NULL, NULL);
	setRegister("incrOptimizeAnalysis", SET_BOOL, &incrOptimizeAnalysis, NULL, NULL, NULL, NULL);
	setRegister("incrOptimizeNet", SET_BOOL, &incrOptimizeNet, NULL, NULL, NULL, NULL);
	//setRegister("incrPressureReduction", SET_FLOAT, &incrPressureRedux, NULL, NULL, NULL, NULL);
	//setRegister("incrFreezeIncoming", SET_BOOL, &incrFreezeIncoming, NULL, NULL, NULL, NULL);
	setRegister("incrUnfreezeThreshold", SET_FLOAT, &incrUnfreezeThreshold, NULL, NULL, NULL, NULL);
	setRegister("incrRetryUnfrozen", SET_BOOL, &incrRetryUnfrozen, NULL, NULL, NULL, NULL);
	setRegister("incrReactivateEdges", SET_BOOL, &incrEdgeReactivation, NULL, NULL, NULL, NULL);
	setRegister("incrGoalRedux", SET_FLOAT, &incrFrobbingGoalDecline, NULL, NULL, NULL, NULL);
	setRegister("incrFreezeHeads", SET_BOOL, &incrFreezeHeads, NULL, NULL, NULL, NULL);
	setRegister("incrFreezingLookaheadSize", SET_INT, &incrLookaheadSize, NULL, NULL, NULL, NULL);
	setRegister("incrAutoFinalize", SET_BOOL, &incrAutoFinalize, NULL, NULL, NULL, NULL);
	setRegister("incrUseFrontierPredicate", SET_BOOL, &incrUseFrontierPredicate, NULL, NULL, NULL, NULL);
	setRegister("alwaysShowAllSolutions", SET_BOOL, &alwaysShowAllSolutions, NULL, NULL, NULL, NULL);
	setRegister("incrTaggerFeedback", SET_BOOL, &incrTaggerFeedback, NULL, NULL, NULL, NULL);
	setRegister("incrTaggerFeedbackDelay", SET_INT, &incrTaggerFeedbackDelay, NULL, NULL, NULL, NULL);
	setRegister("incrTaggerFeedbackRetag", SET_BOOL, &incrTaggerFeedbackRetag, NULL, NULL, NULL, NULL);
	setRegister("initVNsUnused", SET_BOOL, &initVNsUnused, NULL, NULL, NULL, NULL);
	setRegister("incrTaggerLookaheadSize", SET_INT, &incrTaggerLookaheadSize, NULL, NULL, NULL, NULL);

	setRegister("freezing-level", SET_ENUM,&incrFreezeLevel,NULL,NULL,NULL,
			"off", FLTnone,
			"simulated", FLTsimulate,
			"on", FLTfull,
			NULL);
	setRegister("incrFreezingHeuristics", SET_ENUM,&incrFreezingHeuristics,NULL,NULL,NULL,
				"normal", FHconflictFreeProjective,
				"radical", FHradical,
				"semiRadical", FHnotSoRadical,
				"lookahead", FHlookahead,
				NULL);

	labelExceptionProperty = strRegister("LabelExceptions");
	lexemeAttributeExceptionProperty = strRegister("LexemeAttributeExceptions");
	//labelExceptions = inputReadListFromFile(strCat(cdgfolder,"labelException.cfg"));
	//lexemeAttributeExceptions = inputReadListFromFile(strCat(cdgfolder,"lexemeAttributeExceptions.cfg"));
	//incrFluctuatingConstrSections = inputReadListFromFile(strCat(cdgfolder,"fluctuating.cfg"));
	//incrFluctuatingConstrNonspec = inputReadListFromFile(strCat(cdgfolder,"fluctuatingNonspec.cfg"));

	//incrConstraintsFinalScore= inputReadListFromFile(strCat(cdgfolder,"finalConstraints.cfg"));
	//origNonspecScore = memMalloc(sizeof(Number)*listSize(incrConstraintsFinalScore));
}

/**
 * Allocates the IncrementalStep
 * fields still need to be set
 */
IncrementalStep incrStepNew() {
	IncrementalStep step = (IncrementalStep) memMalloc(sizeof (IncrementalStepStruct));
	step->deletedLVs= listNew();
	step->deletedLexemes = listNew();
	step->evalBinary = 0;
	step->evalUnary  = 0;
	step->frozenLVs= listNew();
	step->frozenLexemes = listNew();
	step->frozenLvCount = vectorNew(4);
	vectorSetAllElements(step->frozenLvCount,0);
	step->goal=0.0;
	step->iteration=0;
	step->lexemesTotal= 0;
	step->lvalsTotal= 0;
	step->mode = "";
	step->parse = NULL;
	step->patchAttempts =0;
	step->previousStep = NULL;
	step->problem = NULL;
	step->profiling = listNew();
	step->reanalysed = FALSE;
	step->virtualNodeMatch= FALSE;
	step->repairAttempts = listNew();
	step->returnedLVs = listNew();
	step->returnedLexemes = listNew();
	step->score = 0.0;
	step->stableStructures = listNew();
	step->successfulPatches = 0;
	step->timeNeeded =0;
	step->unfrozenLVs = listNew();
	step->unfrozenLexemes = listNew();
	step->unremovedConflicts= listNew();
	step->unremovedEstablishedConflicts = listNew();
	step->word = "";
	step->slotFrozen = bvNew(0);
	step->lvCountTotal=0;
	step->lvCountBuild=0;
	step->lvCountDeleted=0;
	step->lvCountPostponed=0;
	step->nrTokens=0;
	step->selectedTags = listNew();
	step->killedByTimeLimit=FALSE;
	bvSetAllElements(step->slotFrozen,FALSE);
	step->domainFrozen = bvNew(0);
	bvSetAllElements(step->domainFrozen,FALSE);
	return step;
}

/*
 * Constructor allocating an IncrSubtree
 */
IncrSubtree incrSubtreeNew() {
	IncrSubtree ret= (IncrSubtree) memMalloc(sizeof(IncrSubtreeStruct));
	return ret;
}

/*
 * Constructor allocating a Protoedge
 */
ProtoEdge incrProtoEdgeNew() {
	ProtoEdge ret = (ProtoEdge) memMalloc(sizeof(ProtoEdgeStruct));
	return ret;
}

/*
 * deallocate a protoedge, but not the members that are only pointers to existing objects
 */
void incrProtoEdgeDelete(ProtoEdge pe) {
	if(pe->labels)
		memFree(pe->labels);
	memFree(pe);
}

/*
 * make lattice from list of words
 */
Lattice makeLattice(List words, String name) {
	cdgPrintf(CDG_INFO, "building lattice\n");
	Lattice curLat= (Lattice) memMalloc(sizeof (LatticeStruct));
	curLat->id = strPrintf("wordgraph%d", listSize(words)-1);
	curLat->filename = name;
	curLat->lineNo = 0;
	curLat->arcs= listNew();
	curLat->lookahead = listNew();

	int i = 0; List l;
	for (l = words; l; l = l->next) {
		Arc arc = newArc(i, i + 1, i,
				strRegister(l->item), 0.0, NULL, 0, 0);
		curLat->arcs = listAppendElement(curLat->arcs, arc);
		i++;
	}

	inputScanLattice(curLat);
	return curLat;
}

/*
 * allocates a new measurepoint
 * and sets name and time
 * no need to change it afterwards
 */
ProfilingMeasurePoint profilingMPointNew(String name, unsigned long time) {
	ProfilingMeasurePoint ret= (ProfilingMeasurePoint) memMalloc(sizeof (ProfilingMeasurePointStruct));
	ret->name=name;
	ret->time=time;
	return ret;
}

/************ Helper Methods **************/

/*
 * reads the elapsed time from the given timer (resets it)
 * and attaches a new measure point to the given list
 */
List measureTime(Timer timer, List oldMPs, String name) {
	ProfilingMeasurePoint mp;
	mp=profilingMPointNew(strRegister(name), timerElapsed(timer));
	List ret = listAppendElement(oldMPs,mp);
	timerStart(timer);
	return ret;
}

/**
 * Each constraintnode caches how many values are still valid in this domain
 * incremental expansion will corrupt these values so they have to be recomputed
 */
void incrUpdateNumberCaches(ConstraintNode cNode) {
	int j;
	//int oldNoValidValues =cNode->noValidValues;
	//int oldTotalNumberOfValues= cNode->totalNumberOfValues;
	cNode->noValidValues=0;
	//cNode->totalNumberOfValues=0;
	for(j=0;j<vectorSize(cNode->values);j++) {
		LevelValue lval= vectorElement(cNode->values,j);
		//cNode->totalNumberOfValues++;
		if(!lval->isDeleted) {
			cNode->noValidValues++;
		}
	}
	/*
	if(oldNoValidValues!=cNode->noValidValues)
		cdgPrintf(CDG_WARNING, "CNODE %s_%s: had to update noValidValues %d -> %d\n", cNode->gn->arc->word,cNode->level->id,oldNoValidValues,cNode->noValidValues);
	if(oldTotalNumberOfValues!=cNode->totalNumberOfValues)
		cdgPrintf(CDG_WARNING, "CNODE %s_%s: had to update totalNumberOfValues %d -> %d\n", cNode->gn->arc->word,cNode->level->id,oldTotalNumberOfValues,cNode->totalNumberOfValues);
	*/
}

/**
 * resets limits and delete status of all levelvalues in the constraintnode
 * returns a list of undeleted (i.e. formerly deleted) lvals
 */
List undeleteLVs(ConstraintNode cNode, Boolean freeze) {
	int j;
	List ret = listNew();
	for(j=0;j<vectorSize(cNode->values);j++) {
		LevelValue lval= vectorElement(cNode->values,j);
		if(incrResetLimits) {
			lval->limit=lval->score;
		}
		if(lval->isDeleted)
			ret= listAppendElement(ret,lval);
		if(freeze) {
			lval->isDeleted = lval->isFrozen;
		} else {
			lval->isDeleted = FALSE;
		}
	}
	return ret;
}

/*
 * marks all lexemes in the lexemgraph as not deleted,
 * except those marked as frozen, if freeze is set
 */
void undeleteLexemes(LexemGraph lg, Boolean freeze) {
	int i;

	if(incrResetLimits) { // only resetting lexem limits here, for levelvalues see undeleteLVs
		//cdgPrintf(CDG_INFO, "resetting lexem limits \n");
		for(i=0; i< vectorSize(lg->nodes); i++) {
			LexemNode ln = vectorElement(lg->nodes,i);
			//cdgPrintf(CDG_INFO, "%s : %.2f \n",ln->lexem->description,ln->limit);
			ln->limit= 1.0;
		}
	}

	bvSetAllElements(lg->isDeletedNode,FALSE);

	arrayDelete(lg->distance);
	lg->distance=NULL;

	lgNewFinal(lg);

	if(freeze) {
		for(i= 0 ; i < bvSize(lg->isFrozenNode);i++) {
			bvSetElement(lg->isDeletedNode,bvElement(lg->isFrozenNode,i),i);
		}
	}
}

/**
 * undoes all lexem and edge deletion done by freezing,
 * and resets the problem regarding limits
 */
void unfreezeProblem(Problem P) {
	int i; List l;

	// unfreeze lvs
	Vector lvs= P->net->values;
	for(i=0;i<vectorSize(lvs);i++) {
		LevelValue lv = vectorElement(lvs,i);
		lv->isFrozen= FALSE;
	}

	// undelete lvs
	Vector nodes = P->net->nodes;
	for(i=0;i<vectorSize(nodes);i++) {
		ConstraintNode cn = vectorElement(nodes,i);
		undeleteLVs(cn,FALSE);
	}

	// unfreeze lexemes
	bvSetAllElements(P->lg->isFrozenNode,FALSE);

	// undelete lexemes
	undeleteLexemes(P->lg, FALSE);

	// reset problem global values
	P->globalMaximum = 1.0;
	P->globalMinimum = 0.0;
	listDelete(P->unremovable);
	P->unremovable = listNew();

	// rebuild formerly skipped LVs
	for(i=0;i<vectorSize(nodes);i++) {
		ConstraintNode cn = vectorElement(nodes,i);
		for(l= cn->frozenProtoedges;l; l=l->next) {
			ProtoEdge pe = l->item;
			if(pe->allLabels) {
				cnBuildLevelValues(cn,pe->level,pe->modifier, pe->modifiee);
			} else {
				cnBuildLevelValuesWithFreezing(cn,pe->level,pe->modifier, pe->modifiee,
						FALSE,FALSE,NULL,pe->labels);
			}
			incrProtoEdgeDelete(pe);
		}
		listDelete(cn->frozenProtoedges);
		cn->frozenProtoedges = listNew();

		// recompute cached counts
		incrUpdateNumberCaches(cn);
	}

	// finalize it
	P->net = cnBuildFinal(P->net,TRUE);

	// build domains
	if (P->domains) {
		for (i = 0; i < P->nrSlots; i++)
			listDelete(P->domains[i]);
		memFree(P->domains);
		P->domains = NULL;
	}
	P->domains = (List *) memMalloc(sizeof (List) * P->nrSlots);
	for (i = 0; i < P->nrSlots; i++) {
	  P->domains[i] = NULL;
	}
	importLVs(P, 0);
}

/*
 * returns true if
 * - the section of the constraint is contained in the list of fluctuating
 *   sections
 * - the regent would be nonspec and constraint or its section is
 *   contained in the list of constraints fluctuating fluctuating regarding
 *   nonspec
 *
 * otherwise or if fluctuating score is disabled, it return false
 */
Boolean incrConstraintIsFluctuating(Constraint c, Boolean isNonspec) {
	if(!incrScoreFluctuation)
		return FALSE;

	/*if(listContains(incrFluctuatingConstrSections,c->section->id))
		return TRUE;
	if(isNonspec &&
			(listContains(incrFluctuatingConstrNonspec,c->section->id) ||
			 listContains(incrFluctuatingConstrNonspec,c->id)))
		return TRUE;*/
	if(listContains(c->attributes, inputConstraintFluctuating)) {
		return TRUE;
	}
	if(isNonspec && listContains(c->attributes,inputConstraintFluctuatingNonspec)) {
		return TRUE;
	}
	return FALSE;
}


/*
 * Prompts the user for a string token,
 * prompt and instruction text can be configured
 */
String promptForNextWord(String prompt, String instructions) {
	char *bufptr;
	char buffer[BUFFERSIZE];
	// from terminal
	cdgPrintf(CDG_INFO, "%s\n", instructions);
	cdgPrintf(CDG_INFO, "%s> ", prompt);
	cdgFlush();
	cdgGetString(buffer, BUFFERSIZE);

	// remove trailing whitespace
	while (isspace((int)buffer[strlen(buffer) - 1])) {
		buffer[strlen(buffer) - 1] = '\0';
	}

	// remove leading whitespace
	bufptr = buffer;
	while (isspace((int)*bufptr)) {
		bufptr++;
	}
	//cdgPrintf(CDG_INFO, "typed: %s\n", bufptr);
	return strRegister(bufptr);
}

ChunkIncrement chunkIncrementNew(String string, Boolean tokenize) {
	ChunkIncrement ret = (ChunkIncrement) memMalloc(sizeof (ChunkIncrementStruct));
	ret->string= string;
	ret->words= listNew();
	if(tokenize) {
		String copy = strCopy(string);
		String token = strtok( copy, " ");
		while( token != NULL ) {
			ret->words = listAppendElement(ret->words,token);
		    token = strtok( NULL, " " );
		}
		ret->length = listSize(ret->words);
	} else {
		ret->words = listPrependElement(listNew(),string);
		ret->length =1;
	}
	return ret;
}

/**
 * returns a new chunk made from a vector of strings
 */
ChunkIncrement chunkIncrementNew2(Vector strings) {
	ChunkIncrement ret = (ChunkIncrement) memMalloc(sizeof (ChunkIncrementStruct));
	String s, word;
	int i;

	ret->words= listNew();
	s= "";

	for(i = 0; i< vectorSize(strings); i++) {
		word= vectorElement(strings, i);
		ret->words = listAppendElement(ret->words, word);
		s = strAppend(s, word, " ", NULL);
	}
	ret->string =s;
	ret->length = vectorSize(strings);

	return ret;
}


/*
 * helper to
 * extract some result and profiling information from the given Problem to the given Step
 *
 */
void getInfoFromProblem(IncrementalStep step, Problem P) {
	step->patchAttempts = P->patchAttempts;
	step->successfulPatches = P->successfulPatches;
	step->parse= analysis2parse(P->best);
	cdgExecHook(HOOK_PARTIALRESULT, step->parse);
	parseRegister(step->parse);

	step->repairAttempts= P->repairAttempts;
	step->score= P->best->b->hard?0.0:P->best->b->soft;
	step->unremovedConflicts = listNew();
	step->evalUnary= P->net->evalUnary;
	step->evalBinary= P->net->evalBinary;
	step->lexemesTotal = P->nrLexemes;
	step->lvalsTotal   = P->nrLVs;
	step->frobbingSteps = P->steps;
	step->problem= P;
	step->lvCountTotal=P->net->lvCountTotal;
	step->lvCountBuild=P->net->lvCountBuild;
	step->lvCountDeleted=P->net->lvCountDeleted;
	step->lvCountPostponed=P->net->lvCountPostponed;
	step->killedByTimeLimit= P->killedByTimelimit;
	List tmp = P->unremovable;
	while(tmp) {
		ConstraintViolation cv = tmp->item;
		tmp= tmp->next;
		// cvs will be deleted later, so extract info to strings right here
		step->unremovedConflicts= listAppendElement(step->unremovedConflicts,strRegister(cvToString(cv,TRUE)));
	}
}

/*
 * reevaluate certain constraints for the lv
 */
void reEvalFluctuatingConstraints(LevelValue lv,ConstraintNet net) {
	List l;
	//int oldScore = lv->score;
	lv->score = lv->nonFluctuatingScore; // remove fluctuating penalty
	for(l= inputCurrentGrammar->hashedFlUCs[inputUCIndex(lv->level->no,getDir(lv))];l;l=l->next) {
		Constraint c = l->item;
		lv->constraints = listDeleteElement(lv->constraints, c);
		if(!evalUnaryConstraint(c,net,NULL,lv)) {
			lv->constraints = listPrependElement(lv->constraints,c);
			lv->score *= c->penalty;
		}
	}
	if(lv->regents== NONSPEC) {
		for(l= inputCurrentGrammar->hashedFlUCsNonspec[inputUCIndex(lv->level->no,getDir(lv))];l;l=l->next) {
			Constraint c = l->item;
			lv->constraints = listDeleteElement(lv->constraints, c);
			if(!evalUnaryConstraint(c,net,NULL,lv)) {
				lv->constraints = listPrependElement(lv->constraints,c);
				lv->score *= c->penalty;
			}
		}
	}
	//if(significantlyGreater(oldScore,lv->score) || significantlyGreater(lv->score, oldScore))
	//	cdgPrintf(CDG_INFO,"INFO: a lv's score changed due to fluctuating penalties");
}


/*
 * propagtes a found-conflict-mark up the tree
 */
void propagateConflictHelper(IncrSubtree node) {
	if(node->regent) {
		if(node->regent->conflictFree) {
			node->regent->conflictFree= FALSE;
			propagateConflictHelper(node->regent);
		}
	}
}

/*
 * counts all branching to right of the specified position,
 * going on at the regent of the current subtree and his regent up to the top,
 *
 * a position may be given, nodes left of it will be ignored,
 * should only matter in nonprojective situations
 */
int countRightBranches(IncrSubtree node, int position) {
	int i;
	int count =0;
	List tmp;

	// trees directly under nil cannot have any rightbranches above them
	if(!node->regent) { // reached the top
		return 0;
	}

	// count all branches right of the current one directly under the regent of this one
	tmp =node->regent->underlings;
	int last = listSize(tmp);
	int me = listIndex(tmp,node);
	for(i= me+1; i <= last; i++) {
		IncrSubtree other = listNthElement(tmp,i);
		if(((LexemNode) other->root->dependents->item)->arc->from > position) {
			// ignore those too far to the left
			count ++;
		}
	}

	// count for the regent, but keep the given position, left of it does not count
	count +=
		countRightBranches(node->regent, position);

	return count;
}

/*
 * launches frobbing for a problem and takes care of setting some frobbing variables
 */
void launchFrobbing(Problem P, Number goal){
	frobbingGoal = goal;
	frobResetStepcount();
	combined(P);
	frobbingGoal = 0.0;
}

/*
 * returns if the best analysis of a problem is bad relative to an old score
 * this is the case if
 * - there are hard conflicts remaining
 * - score fell by a factor lower than a configurable threshold
 */
Boolean resultTooBad(Problem P, Number oldScore) {
	if(P->best->b->hard)
		return TRUE;
	if(oldScore==0.0)
		return FALSE;
	Number newScore = P->best->b->soft;
	if(newScore/oldScore < incrUnfreezeThreshold)
		return TRUE;

	return FALSE;
}


/**
 * searches for subtrees in the current analysis fulfilling certain criteria.
 * the criteria are configurable via flags.
 *
 * Substructures may not
 * INCR_SUBSTRUC_CONFLICTS:  contain LVs participating in a (nonignorable) conflict
 * INCR_SUBSTRUC_NONSPEC:    contain LVs attached directly to nonspec
 * INCR_SUBSTRUC_NIL:        contain LVs attached directly to nil
 * INCR_SUBSTRUC_RIGHT_BRANCHES: be the rightmost (newest) attachement at their attachmentpoint,
 * 		branches more to the right can be inherited from above
 * INCR_SUBSTRUC_LEAVES: consist of a single LV, i.e be a leave
 *
 * Subtrees may not be part of another subtree in this selection
 * (i.e. only maximal stable subtrees)
 */
List searchStableSubstructures(IncrementalStep step, int flags, List levels) {
	cdgPrintf(CDG_INFO, "search for stable substructures\n");
	int i;
	List tmp;
	Vector edges= step->problem->best->slots;
	List ret = listNew();
	List edgeManagement = listNew(); // list of subtreestructs

	//cdgPrintf(CDG_INFO, "edges: %d\n", vectorSize(edges));

	// for all edges initialize a subtreestructure and evaluate LV based criteria
	// such as existence of conflicts
	for(i=0;i<vectorSize(edges);i++) {
		LevelValue lv = vectorElement(edges,i);

		if(!lv)
			continue;

		// skip Levels not included in the levels list
		if(!listContains(levels, lv->level))
			continue;

		// List conflicts = lv->constraints;
		List conflicts= listNew();
		Boolean free = TRUE; // free of serious conflicts
		if(flags & INCR_SUBSTRUC_CONFLICTS) {
			List allConflicts= step->problem->best->conflicts;
			ConstraintViolation cv;
			// search all conflicts ...
			for( ;allConflicts;allConflicts = allConflicts->next) {
				cv = allConflicts->item;
				if(lvIndex(lv) == cv->nodeBindingIndex1 || lvIndex(lv) == cv->nodeBindingIndex2) {
					// ... for conflicts the current lv participates in ...
					if(cv->penalty < frobIgnoreThreshold) {
						// ... and of those only the serious ones
						conflicts = listPrependElement(conflicts,cv);
					}
				}
			}
			if(conflicts)  // there where serious conflicts
				free = FALSE;
		}
		if(flags & INCR_SUBSTRUC_NONSPEC) {
			if(lv->regents == NONSPEC)
				free =FALSE;
		}
		if(flags & INCR_SUBSTRUC_NIL) {
			if(!lv->regents)
				free =FALSE;
		}

		IncrSubtree entry = incrSubtreeNew();
		entry->root =lv;
		entry->conflictFree = free;
		// following vars are evaluated later and just initialized here:
		entry->regent=NULL;
		entry->isHead=FALSE;
		entry->acceptable=FALSE;
		entry->underlings = listNew();
		entry->conflicts = conflicts;

		edgeManagement = listAppendElement(edgeManagement,entry);
	} // end for all edges

	// fill regent fields
	//cdgPrintf(CDG_INFO, "fill regent fields for %d subtrees\n", listSize(edgeManagement));
	tmp = edgeManagement;
	while(tmp) { IncrSubtree node= tmp->item; tmp = tmp->next;
		LevelValue above = edgeAbove(step->problem->best,node->root);
		if(above) {
			List tmp2 = edgeManagement;
			while(tmp2) { IncrSubtree node2= tmp2->item; tmp2 = tmp2->next;
				if(above == node2->root) {
					// prevent circles:
					IncrSubtree nd= node2;
					while(nd) {
						if(nd== node) {
							cdgPrintf(CDG_WARNING,"WARNING: circle in subtree detected! skipping...\n");
							return listNew();
						}
						nd= nd->regent;
					}

					node->regent= node2;
					node2->underlings = listAppendElement(node2->underlings, node);
					break;
				}
			}
		}
	}


	// as we have this structure build up we can use it for some offtopic evaluation
	cdgPrintf(CDG_INFO,"Searching for established conflicts\n");
	tmp = edgeManagement;
	while(tmp) {IncrSubtree node= tmp->item; tmp = tmp->next;
		// we want to use the rightbranch criteria to search for established conflicts
		if(countRightBranches(node,((LexemNode) node->root->dependents->item)->arc->from)) {
			List l = node->conflicts;
			for(;l;l=l->next) {
				String confl = strRegister(cvToString(l->item,TRUE));
				cdgPrintf(CDG_INFO," established:%s\n", confl);
				step->unremovedEstablishedConflicts = listAddUniqueElement(
						step->unremovedEstablishedConflicts,
						confl);
			}
		}
	}

	// propagate the stain of conflict-involvement up the tree
	//cdgPrintf(CDG_INFO, "propagate stain\n");
	tmp = edgeManagement;
	while(tmp) {IncrSubtree node= tmp->item; tmp = tmp->next;
		if(!node->conflictFree)
			propagateConflictHelper(node);
	}

	// collect conflict free subtrees
	//cdgPrintf(CDG_INFO, "collect\n");
	tmp = edgeManagement;
	while(tmp) { IncrSubtree node= tmp->item; tmp = tmp->next;
		Boolean additionalCriteriaFulfilled = TRUE;
		if(flags & INCR_SUBSTRUC_RIGHT_BRANCHES) {
			node->rightBranches = countRightBranches(node,((LexemNode) node->root->dependents->item)->arc->from);
			if(node->rightBranches == 0 )
				additionalCriteriaFulfilled = FALSE;
		}
		if(flags & INCR_SUBSTRUC_LEAVES) {
			if(listSize(node->underlings) ==0)  // leave nodes are bad subtrees
				additionalCriteriaFulfilled = FALSE;
		}
		// ----------------
		if(node->conflictFree && additionalCriteriaFulfilled) {
			node->acceptable=TRUE;
		}
	} // end for all subtrees

	// mark heads of conflict free subtress
	tmp = edgeManagement;
	//cdgPrintf(CDG_INFO, "mark heads for %d subtrees\n", listSize(edgeManagement));
	while(tmp) { IncrSubtree node= tmp->item; tmp = tmp->next;
		if(node->acceptable && (!node->regent || !node->regent->acceptable)) {
			node->isHead= TRUE;
			ret= listAppendElement(ret,node);
		}
	}
	cdgPrintf(CDG_INFO, "done, %d stable structures found\n", listSize(ret));

	return ret;
}

/**
 * Freezes LVs in the given domain.
 * It spares the selected LV
 * and all LVs that share with the selected one an unfrozen, undeleted Lexeme at each end
 *
 * returns a list of frozen LVs
 */
List freezeLVsInDomain(Problem P, int domain, LevelValue selected) {
	List ret = listNew();
	// mark the LVs in the same domain as frozen
	List tmp2= P->domains[domain];
	while(tmp2) { LevelValue lv = tmp2->item; tmp2=tmp2->next;
		if(lv == selected)  // don't freeze the selected one
			continue;

		if(lvEquals(selected,lv)) { // this exception is needed to allow later case changes etc.
			// lv is equal to the selected one in from, to and label,
			// if any of it's lexemes on both ends are still unfrozen, don't freeze it.
			Boolean activeDown= FALSE;
			Boolean activeUp= FALSE;
			List l;
			LexemGraph lg = P->lg;
			for(l= lv->dependents; l; l=l->next) { //look for active dependent lexemes
				LexemNode ln = l->item; int no = ln->no;
				if(!bvElement(lg->isFrozenNode,no) && !bvElement(lg->isDeletedNode,no)) {
					activeDown = TRUE; break;
				}
			}
			for(l= lv->regents; l; l=l->next) { // look for active regent lexemes
				LexemNode ln = l->item; int no = ln->no;
				if(!bvElement(lg->isFrozenNode,no) && !bvElement(lg->isDeletedNode,no)) {
					activeUp = TRUE; break;
				}
			}
			if(activeUp && activeUp) {
				//don't freeze it
				continue;
			}
		}
		lv->isFrozen = TRUE;
		ret= listPrependElement(ret, lv);
	}

	return ret;
}

/**
 * returns a list of LVs not in the current analysis
 * and concurrating to those in a stable subtree
 * they are not only returned but marked as frozen,
 * if incrfreeze is set, the frozen mark will exclude it from further processing
 */
List freezeLVsInSubtree(IncrSubtree st,Problem P,IncrementalStep step) {
	List ret= listNew();
	List tmp = st->underlings;

	// freezecounts
	int incoming =0;
	int outgoing = 0;

	// mark the slot frozen for incomming LVs
	int slot= lvIndex(st->root);
	if(incrFreezeIncoming && incrFreezingHeuristics == FHconflictFreeProjective)
		bvSetElement(step->slotFrozen,TRUE,slot);
	if(incrFreezeHeads)
		bvSetElement(step->domainFrozen,TRUE,slot);

	while(tmp) { IncrSubtree underling = tmp->item; tmp=tmp->next;
		LevelValue selected = underling->root;
		int domain = lvIndex(selected);
		bvSetElement(step->domainFrozen,TRUE,domain);

		List l = freezeLVsInDomain(P,domain,selected);
		outgoing += listSize(l);
		ret = listAppendList(ret,l);

		ret = listAppendList(ret,freezeLVsInSubtree(underling,P,step));
	}

	// above rule only freezes lvs inside the domains
	// following rule prevents other LVs to be added to the subtree
	if(incrFreezeIncoming && incrFreezingHeuristics == FHconflictFreeProjective) {
		int i;
		for(i=0; i < vectorSize(P->net->values); i++) { // for all lvs
			LevelValue lv = vectorElement(P->net->values,i);
			if(lv->level == st->root->level && lvIndex(st->root) == lvIndexAbove(lv)) { // slot frozen?
				Boolean b= TRUE;
				for(tmp = st->underlings; tmp; tmp= tmp->next) {
					if(lvIndex(((IncrSubtree)tmp->item)->root) == lvIndex(lv)) {
						b= FALSE;
						break;
					}
				}
				if(!b) {
					// aktuelle Kante ist Teil des ausgew�hlten baumes,
					// wird damit also von der ersten regel behandelt
					continue;
				}

				ret = listPrependElement(ret, lv);
				lv->isFrozen = TRUE;
				incoming++;
			} // end if slot frozen
		} // end for all lvs
	} // end if freezeincoming

	Vector v = step->frozenLvCount; // vector containing ints instead of pointers
	int oldOut = (int)vectorElement(v,0);
	int oldIn  = (int)vectorElement(v,1);
	vectorSetElement(v,(Pointer)oldOut+outgoing,0);
	vectorSetElement(v,(Pointer)oldIn+incoming,1);

	return ret;
}


/**
 * returns a bytevector indicating which slots are new in the given step
 * compared to the previous one
 */
ByteVector recentSlots(IncrementalStep step) {

	int nrSlots= step->nrTokens;

	// initialize as all recent
	ByteVector result= bvNew(nrSlots);
	bvSetAllElements(result,TRUE);

	// set those indices to false, that are already present in the step before that
	if(step->previousStep) {
		int i;
		for(i=0; i< step->previousStep->nrTokens; i++) {
			bvSetElement(result,FALSE,i);
		}
	}

	return result;
}

/**
 * mark all indices of slots that have lexemes with different pos tags selected in the analysis
 */
ByteVector changedTagsSlots(IncrementalStep step) {

	int nrSlots= step->nrTokens;

	// initialize all as unchanged
	ByteVector result= bvNew(nrSlots);
	bvSetAllElements(result,FALSE);

	// set those indices to true, that  have lexemes with different pos tags selected in the analysis
	if(step->previousStep) {
		int i;
		for(i=0; i< step->previousStep->nrTokens; i++) {
			int index = 0; // TODO
			LexemNode oldLn = ((LevelValue)vectorElement(step->previousStep->parse->LVs, index))->dependents->item;
			LexemNode newLn = ((LevelValue)vectorElement(step->parse->LVs, index))->dependents->item;
			String oldTag= leGetValue(oldLn->lexem, inputCurrentGrammar,"cat");
			String newTag= leGetValue(newLn->lexem, inputCurrentGrammar,"cat");
			if(!strcmp(oldTag,newTag)) {
				bvSetElement(result,TRUE,i);
			}
		}
	}

	return result;
}

/**
 *
 */
List freezeLexemesFromList(Problem P, List lexemes ,LexemNode selected) {
	List ret= listNew();
	List exceptions = inputGetProperty(inputCurrentGrammar,lexemeAttributeExceptionProperty);
	while(lexemes) { LexemNode ln = lexemes->item; lexemes = lexemes->next;
		if(bvElement(P->best->isUnselectedLexeme, ln->no )) {
			if(leCompareWithExceptions(inputCurrentGrammar,selected->lexem,ln->lexem,FALSE,exceptions)>0) {
				//if(incrFreeze)
				bvSetElement(P->net->lexemgraph->isFrozenNode,TRUE,ln->no);
				ret= listPrependElement(ret,ln);
			} else {
				//cdgPrintf(CDG_INFO,"didn't freeze lexeme %s because of case\n", ln->lexem->description);
			}
		}
	}
	return ret;
}

/**
 * returns a list of Lexemes not in the current analysis concurrating to those in a stable subtree
 * they are not only returned but marked as frozen, if incrfreeze is set frozen mark will exclude it from processing
 * lexemes that differ only in attributes included in the lexemeAttributeExceptions list
 * will still be available
 */
List freezeLexemes(IncrSubtree st,Problem P) {
	List lexemes = ((LexemNode)st->root->dependents->item)->grapheme->lexemes;
	LexemNode selected = selectedLexeme(P->best,lexemes);

	List ret= freezeLexemesFromList(P,lexemes,selected);

	List tmp = st->underlings;
	while(tmp) { IncrSubtree underling = tmp->item; tmp=tmp->next;
		ret = listAppendList(ret,freezeLexemes(underling,P));
	}

	return ret;
}

// marks all LVs except those in the last 'lookhead' slots per level
void freezeByFixedLookaheadSize(IncrementalStep step, int lookahead) {
	Problem P = step->problem;
	int levels = P->nrLevels;
	int slots  = P->nrSlots;

	int d;
	for(d=0;d<(slots- lookahead*levels);d++) {
		LevelValue selectedLV = slotFiller(P->best,d);
		if(selectedLV->regents==NONSPEC) // don't freeze nonspec LVs
			continue;

		List lexemes = ((LexemNode)selectedLV->dependents->item)->grapheme->lexemes;
		LexemNode selectedLex = selectedLexeme(P->best,lexemes);

		freezeLexemesFromList(P,lexemes,selectedLex);

		bvSetElement(step->domainFrozen,TRUE,d);
		freezeLVsInDomain(P,d,selectedLV);
	}

}


/*
 * sets penalty of the constraint globaly
 * and stores the old penalty
 * TODO does not support penalty terms yet
 */
void setPenalty(Number newPenalty, Constraint c) {
	c->penaltyCache = c->penalty;
	c->penalty = newPenalty;
}

/*
 * resets constraint penalty to old value
 */
void resetPenalty(Constraint c) {
	c->penalty = c->penaltyCache;
}

/*
 * prepares the final step of an incremental parsing process
 * has some global side effects, call postpareFinalStep() to revert them
 */
void prepareFinalStep(Problem P) {
	int i;
	List l;
	/*for(i=0; i<vectorSize(P->net->values);i++) {
		LevelValue lv = vectorElement(P->net->values,i);
		if(lv->regents == NONSPEC) {
			lv->score=0.0;
			lv->nonFluctuatingScore=0.0;
		}
	}*/

	l= incrTemporarilyAllowedConstraints;
	while(l) { Constraint c = l->item; l= l->next;
		setPenalty(0.0,c);
		for(i=0; i<vectorSize(P->net->values);i++) {
			LevelValue lv = vectorElement(P->net->values,i);
			if(listContains(lv->constraints,c)) {
				lv->score=0.0;
				if(!incrConstraintIsFluctuating(c, lv->regents==NONSPEC ||
						lvGetDependentGN(lv)->isVirtual || (lv->regents && lvGetRegentGN(lv)->isVirtual) )) {
					lv->nonFluctuatingScore=0.0;
				}
			}
		}
	}
	l= incrTempPenalizedConstraints;
	while(l) { Constraint c = l->item; l= l->next;
		setPenalty(1.0,c);
		for(i=0; i<vectorSize(P->net->values);i++) {
			LevelValue lv = vectorElement(P->net->values,i);
			if(listContains(lv->constraints,c)) {
				lv->score=lv->score/c->penalty; // TODO does not work for constraints with penalty 0.0
				if(!incrConstraintIsFluctuating(c, lv->regents==NONSPEC ||
						lvGetDependentGN(lv)->isVirtual || (lv->regents && lvGetRegentGN(lv)->isVirtual) )) {
					lv->nonFluctuatingScore=lv->nonFluctuatingScore/c->penalty;
				}
			}
		}
	}
}

void postpareFinalStep() {
	// reset scores of constraints with a different final score
	List l= incrTemporarilyAllowedConstraints;
	for(;l;l=l->next) {
		Constraint c = l->item;
		resetPenalty(c);
	}
}

/**
 * returns, if one of the given lexemes is a selected one
 */
Boolean containsSelected(List lexemes, ByteVector isUnselected) {
	List l = lexemes;
	LexemNode ln;

	while(l) {
		ln = l->item; l=l->next;
		if(!bvElement(isUnselected,ln->no)) {
			return TRUE;
		}
	}
	return FALSE;
}

/**
 * insert slot,
 * take care that all virtual nodes stay in the back
 */
int insertSlot(LevelValue lv, Vector slots) {
	if(!useVirtualNodes || gnIsVirtual(((LexemNode)lv->dependents->item)->grapheme)) {
		return vectorAddElement(slots,lv); // just add it at the end
	} else {
		int n=0;
		for(n=0;n<vectorSize(slots);n++) {
			LevelValue lv2= (LevelValue) vectorElement(slots,n);
			if(!lv2)
				continue;
			if(gnIsVirtual(lvGetDependentGN(lv2))) {
				break; // first virtual node found, insert here
			}
		}
		vectorInsertElement(slots,lv,n);
		return n;
	}

}


/******************* Extenders ********************/

/*
 * binds another slot in the given analysis
 * on every level a levelvalue is selected so that it binds a lexem as regent that is
 * already part of the selected path through the wordgraph in the analysis
 * if there is no such lexem NIL (and NONSPEC, if used) will always be available,
 * so it will always succed
 *
 * after the first level there will be an lexem selected as dependent as well,
 * so the values on the other levels have to respect this
 * this will be no problem in the standard setting where SYN is the first level
 * and the others are unproblematic.
 *
 * In other settings an error might occur!
 *
 * due to virtual nodes not yet pushed back, newSlots may not be read!
 */
void bindAnotherSlot (Analysis A, ByteVector newIsUnselected, Vector newSlots, int timepoint) {
	int lvl;
	int lastpos=0;
	LexemNode selectedLexem = NULL;
	Problem P = A->P;

	for(lvl=0; lvl<P->nrLevels;lvl++) { // fuer alle Level...

		if(!((Level)listNthElement(inputCurrentGrammar->levels,lvl+1))->useflag) {
			// skip unused levels
			vectorInsertElement(newSlots,NULL,lastpos+1);
			lastpos++;
			continue;
		}

		List lvs = P->domains[P->nrLevels * timepoint + lvl];
		Boolean acceptable = FALSE;
		LevelValue lv= NULL;

		if(!selectedLexem) { // selected lexem== NULL,
			// nur fuer den ersten level der Fall, nachfolgende levels muessen zusaezlich darauf achten, dass das hier gew�hlte lexem benutz wird
			//durchsuche alle Levelvalues der neuen domaene
			while(lvs) {
				//cdgPrintf(CDG_INFO, "lvs left: %d\n", listSize(lvs));
				// waehle einen Levelvalue aus...
				lv = lvs->item;
				lvs = lvs->next;

				if(lv->dependents == NONSPEC) {
					cdgPrintf(CDG_INFO, "WARNING: NONSPEC as dependent!\n");
					continue;
				}

				// ...und schaue fuer die regenten, ob ein entsprechndes lexem verfuegbar ist
				// in diesem fall ist der levelvalue ein zulaessiger  (wenn auch nicht unbedingt linguistisch sinnvoller) wert
				List lns; // lexemnodes
				if(!lv->regents || lv->regents== NONSPEC) { // no need to check if regent LNs are selected, as there are none
					if(lv->score> 0) {
						acceptable = TRUE;
					} else { // better not take a lv with score 0.0
						continue;
					}
				} else {
					for(lns = lv->regents; lns; lns=lns->next) {
						LexemNode ln = (LexemNode) lns->item;
						//cdgPrintf(CDG_INFO, "current node: %s\n", ln->arc->word);
						if(!bvElement(newIsUnselected,ln->no)) {
							acceptable = TRUE;
							//cdgPrintf(CDG_INFO, "acceptable lv foud\n");
							//cdgPrintf(CDG_INFO, "\t %s: %s -- %s -> %s \n", lv->level->id, ((LexemNode)lv->dependents->item)->lexem->description,lv->label,((LexemNode)lv->regents->item)->lexem->description);
							break;
						}
					}
				}
				if(acceptable) { // akzeptabler Startwert fuer die neue domaene gefunden
					break;
				} else {
					//cdgPrintf(CDG_INFO, "need to search some more ...\n");
				}
			} // end while(lvs)

			if(!acceptable) {
				// wenn keiner passt, nimm den ersten
				lv = P->domains[P->nrLevels * timepoint + lvl]->item;
			}
			lastpos= insertSlot(lv,newSlots);
			selectedLexem= lv->dependents->item; // lexem fuer spaeter merken
			bvSetElement(newIsUnselected,FALSE,selectedLexem->no);

		} else { // ein lexem wurde bereits in einem anderen level ausgewaehlt
			// durchsuche alle levelvalues nach Kandidaten
			while(lvs) {
				lv = lvs->item;
				lvs=lvs->next;
				if(listContains(lv->dependents,(Pointer)selectedLexem)) {
					if(lv->regents == NULL || lv->regents == NONSPEC ||
							containsSelected(lv->regents,newIsUnselected)) {
						acceptable=TRUE;
						break;
					}
				}
			}
			if(acceptable) {
				lastpos= insertSlot(lv,newSlots);
				// isUnselected schon gesetzt von anderem level
			} else {
				// TODO geht davon aus, dass die erste ebene die syntaxebene ist und
				//  dass alle weiteren zu jedem lexem ein levelvalue parat haben (wie es bei ref und arg der fall ist)
				cdgPrintf(CDG_ERROR, "ERROR: expected level to be unproblematic regarding lexems, but it was not \n");
			}
		} // end if(selectedLexem)
	} // end fuer alle Levels
}


/*
 * Extends an analysis so that it binds all domains of an extended problem
 */
Analysis extendAnalysis(Analysis A, Problem P, int chunkLength, IncrementalStep step) {
	cdgPrintf(CDG_INFO, "extending Analysis \n");

	Timer timer = timerNew();

	// Deklarationen
	int i;
	Vector newSlots = vectorNew(P->nrSlots);
	Vector oldSlots= A->slots;
	ByteVector newIsUnselected= bvNew(P->nrLexemes);
	ByteVector oldIsUnselected= A->isUnselectedLexeme;

	A->P = P;

	// kopiere isUnselected
	bvSetAllElements(newIsUnselected,TRUE);
	for(i=0; i<bvSize(oldIsUnselected);i++) {
		bvSetElement(newIsUnselected,bvElement(oldIsUnselected,i),i);
	}

	// den alten path uebernehmen
	//cdgPrintf(CDG_INFO, "importing old path \n");
	for(i= 0; i< vectorSize(oldSlots);i++) {
		LevelValue lv = vectorElement(oldSlots,i);
		// virtual nodes will be pushed back later when inserting the new part
		// don't read the vector before that!
		vectorAddElement(newSlots,lv);
	}

	step->profiling = measureTime(timer,step->profiling,"  preparation");

	// den letzten Schritt neu hinzufuegen
	int newestTP;
	if(useVirtualNodes) {
		newestTP = P->lg->ve->vStartID;
	} else {
		newestTP = P->maxTimepoint;
	}

	for(i=newestTP-chunkLength; i<newestTP; i++) {
		bindAnotherSlot(A, newIsUnselected, newSlots,i);
	}

	step->profiling = measureTime(timer,step->profiling,"  bind new slots");

	if(useVirtualNodes) {
		// new virtual nodes might need binding, too
		for(i=vectorSize(newSlots)/P->nrLevels;i<P->maxTimepoint;i++) {
			bindAnotherSlot(A, newIsUnselected, newSlots,i);
		}
		step->profiling = measureTime(timer,step->profiling,"  bind new vNodes");
	}

	// new slots safe to read again

	// neue uebernehmen
	A->slots= newSlots;
	A->isUnselectedLexeme= newIsUnselected;

	// alte loeschen
	vectorDelete(oldSlots);
	bvDelete(oldIsUnselected);

	// neuen timepoint in den beruecksichtigten Bereich hinzufuegen
	A->subStop+= chunkLength;

	aJudge(A);

	step->profiling = measureTime(timer,step->profiling,"  judge");

	if(incrOptimizeAnalysis) {
		// optimizeDomains already uses P->current
		// so make sure there is no old Analysis incompatible with the constraintnet there
		P->current=NULL;
		//
		Patch p = optimizeDomains(A,(Pointer)((A->subStop-1)*P->nrLevels),OD_LIMIT|OD_PRUNE|OD_NARROW|OD_INT|OD_ALLOWALL);
		if(p) {
			apply(A,p);
			aJudge(A);
		} else {
			cdgPrintf(CDG_WARNING,"WARNING: optimization of domains not possible!\n");
		}
	}

	step->profiling = measureTime(timer,step->profiling,"  optimize");

	aCheck(A,3);

	step->profiling = measureTime(timer,step->profiling,"  check");

	if(useVirtualNodes && matchVirtualNodes) {
		// evaluate if just replacing one of the new nodes for a virtual one results in a better analysis
		if(chunkLength > 1) {
			for(i=newestTP-chunkLength; i<newestTP; i++) {
				GraphemNode newNode = vectorElement(P->net->lexemgraph->graphemnodes, i);
				// TODO last score might be too harsh of a threshold
				A= aBestMatchingReplacement(A, newNode, step->previousStep->score * virtMatchingFactor);
			}
		} else {
			// with only one chunk this is straight forward;
			GraphemNode newNode = vectorElement(P->net->lexemgraph->graphemnodes, newestTP-1);
			Analysis newA = aBestMatchingReplacement(A, newNode, step->previousStep->score * virtMatchingFactor);
			if(A!= newA) {
				step->virtualNodeMatch= TRUE;
			}
			A= newA;
		}

	}

	step->profiling = measureTime(timer,step->profiling,"  match virtuals");
	timerFree(timer);

	return A;
}

/**
 * push an arc back one position,
 * might increase lg->max
 */
void pushArc(Arc arc, LexemGraph lg) {
	arc->from++;
	arc->to++;
	lg->max = max(lg->max, arc->to);
}

/**
 * extends the given lexemgraph with the given arc, mostly reusing code from
 * lexemgraph.c
 */
LexemGraph extendLexemGraph(LexemGraph lg, Arc arc) {
	//cdgPrintf(CDG_INFO, "extending lexemgraph... \n");

	// extending lattice
	lg->lattice->arcs = listAppendElement(lg->lattice->arcs, arc);
	inputScanLattice(lg->lattice);

	lg->noOfPathsFromStart=NULL;
	lg->noOfPathsToEnd =NULL;

	lgNewIter(lg,arc);

	undeleteLexemes(lg,(incrFreezeLevel == FLTfull));

	return lg;
}

/**
 * Cleanup for after extending the net before frobbing it
 *
 * - check postponed LVs
 * - check fluctuating scores of LVs
 * - calls cnFinal
 */
ConstraintNet postExtendNetCleanup(ConstraintNet net, IncrementalStep step) {
	Timer timer = timerNew();
	List l;
	int i;

	// look at all postponed LVs for if they should be reactivated
	if(step && incrEdgeReactivation) {
		List toBeDeleted = listNew();
		for(l =net->postponedLVs; l; l= l->next) {
			LevelValue lv= (LevelValue)l->item;
			Number newScore = evalUnary(lv,net,NULL,FALSE,NULL,NULL,TRUE);
			if(newScore > 0.0)
				toBeDeleted = listPrependElement(toBeDeleted, lv); // can't delete in iteration
		}
		for(l= toBeDeleted;l;l=l->next) {
			LevelValue lv= (LevelValue)l->item;
			//cdgPrintf(CDG_INFO,"Activate LV: ");
			//lvPrint(CDG_INFO,lv,NULL,1);
			//cdgPrintf(CDG_INFO,"\n");
			cnReactivateLV(net,lv);
		}
		listDelete(toBeDeleted);

		if (step)
			step->profiling = measureTime(timer,step->profiling,"  reeval outside");
	}

	// old LevelValues need to have some of their constraints rechecked:
	for(i=0;i<vectorSize(net->values);i++) {
		LevelValue lv = vectorElement(net->values,i);
		if(!incrFreezeLevel== FLTfull || !lv->isFrozen)
			reEvalFluctuatingConstraints(lv,net);
	}
	if (step)
		step->profiling = measureTime(timer,step->profiling,"  reeval inside");

	// undeleting
	for(i=0; i<vectorSize(net->nodes);i++) {
		ConstraintNode cn= vectorElement(net->nodes,i);
		// undelete resets the deleted flag for all LVs
		List undeleted = undeleteLVs(cn, incrFreezeLevel == FLTfull);

		if(step) {
			// safe formerly deleted lvals for later
			List tmp= undeleted;
			while(tmp) {
				LevelValue lv = tmp->item;
				tmp=tmp->next;
				if(!listContains(step->deletedLVs, lv))
					step->deletedLVs = listAppendElement(step->deletedLVs,lv);
			}
		}
		listDelete(undeleted);

		incrUpdateNumberCaches(cn);
	}

	ConstraintNet ret = cnBuildFinal(net, TRUE);
	cdgPrintf(CDG_INFO, "finished constraintnet \n");

	if (step)
		step->profiling = measureTime(timer,step->profiling,"  finalizing");
	timerFree(timer);

	return ret;
}




/**
 * builds all levelvalues that go towards the new graphemenode,
 * i.e. levelvalues in all the other constraintnodes on the same level
 */
void incrBuildLvsTowardsNewNode(ConstraintNet net, GraphemNode newNode, ByteVector exceptions, Vector freezeCount) {
	LexemGraph lg = net->lexemgraph;
	List labelExceptions = inputGetProperty(inputCurrentGrammar,labelExceptionProperty);
	int k;
	List l;

	int ggs = vectorSize(lg->graphemnodes);
	int frozenLVsOutgoing=0;
	GraphemNode modifier;
	ConstraintNode cNode= NULL;
	//cdgPrintf(CDG_INFO, "   --dist: %d\n", arrayDimension(lg->distance,0)); // NB
	for (l = inputCurrentGrammar->levels; l != NULL; l = l->next) {
	  Level level = (Level) l->item;
	  if (!level->useflag)
	    continue;
	  for (k = 0; k <= ggs; k++) {
		if(k == ggs) {
			modifier = NULL;
		} else {
			modifier = vectorElement(lg->graphemnodes, k);
			int cnPos = cnGaplessIndex(net, level,k);
			cNode = vectorElement(net->nodes,cnPos);
			if(incrFreezeLevel == FLTnone || !exceptions || !bvElement(exceptions,cnPos)) {
				cnBuildLevelValues(cNode, level, modifier, newNode);
			} else {
				// FROZEN!
				Boolean skipFrozen;
				if(incrFreezeLevel == FLTsimulate) {
					skipFrozen= FALSE;
				} else if(incrFreezeLevel == FLTfull) {
					skipFrozen= TRUE;
				} else {
					cdgPrintf(CDG_WARNING,"WARNING: domain unexpectedly frozen!");
					skipFrozen= FALSE;
				}
				frozenLVsOutgoing += cnBuildLevelValuesWithFreezing(
						cNode, level, modifier, newNode, TRUE, skipFrozen, labelExceptions, NULL);
			}
		}
	  }
	}
	if(freezeCount) {
		vectorSetElement(freezeCount,vectorElement(freezeCount,2)+frozenLVsOutgoing,2);
	}
}

/**
 * extend the isBuild array
 * extra care needs to be taken if virtual nodes are present
 */
void extendCNIsBuild(ConstraintNet net) {
	int i,j,k, i2, j2;
	Array oldIsBuild= net->isBuilt;
	LexemGraph lg = net->lexemgraph;

	int firstVirtual = lg->ve ? lg->ve->vStartID : (lg->max);

	net->isBuilt = arrayNew( // all values will be initialized as 0
			lg->max,
			(2+lg->max), // +2 because of NIL and NONSPEC
			listSize(inputCurrentGrammar->levels),
			0);
	for(i=0;i<arrayDimension(oldIsBuild,0);i++) { // from
		for(j=0;j<arrayDimension(oldIsBuild,1);j++) { // to
			for(k=0;k<arrayDimension(oldIsBuild,2);k++) { // levels

				if(i>=firstVirtual) {
					i2= i+1;
				} else {
					i2= i;
				}

				if(j>=firstVirtual) {
					j2 = j+1;
				} else {
					j2= j;
				}
				arraySetElement(net->isBuilt,arrayElement(oldIsBuild,i,j,k),i2,j2,k);
			}
		}
	}
	arrayDelete(oldIsBuild); // we don't need it anymore
}

/**
 * The given constraintnet is extended by the given arc
 * the step is only here to collect profiling information
 *
 *	- extends the lexemgraph
 *	- builds new levelvalues for the new domain
 *	- build  new levelvalues for the old domains leading to the new timeslot
 */
ConstraintNet extendNet(ConstraintNet net, Arc arc, IncrementalStep step) {
	cdgPrintf(CDG_INFO, "extending constraintnet  %s \n", net->id);

	LexemGraph lg= net->lexemgraph;
	int i;
	int lgmax;

	// Timer timer = timerNew();

	if(step) {
		// remember deleted lexemes for later use
		for(i=0;i<bvSize(lg->isDeletedNode); i++) {
			if(bvElement(lg->isDeletedNode,i)) {
				LexemNode ln = vectorElement(lg->nodes,i);
				if(!listContains(step->deletedLexemes,ln))
					step->deletedLexemes= listAppendElement(step->deletedLexemes,ln);
			}
		}
	}

	lgmax= lg->max;

	lg = extendLexemGraph(lg,arc);

	if(! (lg->max > lgmax) ) {
		cdgPrintf(CDG_DEBUG,"ERROR: in extendNet, lexemgraph not extended, aborting \n");
		abort();
	}

	// tag, either with or without feedback
	if(step && incrTaggerFeedback) {
		// let's do the tagging with feedback
		List tagHistory= NULL;
		if(incrTaggerFeedbackRetag) {
			tagHistory = step->previousStep->selectedTags;
		}

		List feedback = taggerCalculateFeedback(step->selectedTags,
				tagHistory, incrTaggerFeedbackDelay);

		cdgPrintf(CDG_DEBUG,"Feedback to Tagger:\n");
		List l= feedback;
		for(;l;l=l->next) {
			cdgPrintf(CDG_DEBUG,"%s\n",l->item);
		}

		// step->taggerScore =
		taggerTag(lg, feedback);
	} else {
		lgTag(lg);
	}

	// either way, call all the other predictors
	lgPredict(lg);

	//step->profiling = measureTime(timer,step->profiling,"  extend lexemgraph");

	// Das Array isBuild kann im Gegensatz zu den Vektoren nicht erweitert werden
	// und muss neu allociert werden,
	extendCNIsBuild(net);

	//step->profiling = measureTime(timer,step->profiling,"  isBuild stuff");

	// cnBuildIter fuegt nur LVs hinzu, die vom neuen gn aus nach oben gehen...
	GraphemNode gn;
	if(useVirtualNodes) {
		gn= vectorElement(lg->graphemnodes,lg->ve->vStartID-1); // there are virtual gns behind the one we search
	} else {
		gn = vectorElement(lg->graphemnodes,vectorSize(lg->graphemnodes)-1);
	}
	if(!step || incrFreezeLevel==FLTnone) {
		cnBuildIter(net, gn,TRUE);
	} else if (incrFreezeLevel== FLTsimulate){
		cnBuildIterWithExceptions(net, gn, TRUE, step->slotFrozen, step->frozenLvCount);
	} else if (incrFreezeLevel== FLTfull) {
		cnBuildIterWithExceptions(net, gn, TRUE, step->slotFrozen, NULL);
	}

	//step->profiling = measureTime(timer,step->profiling,"  cnBuildIter");

	// ... daher muessen die LVs in die andere Richtung getrennt hinzugefuegt werden
	// cdgPrintf(CDG_INFO, "other direction \n");
	if(step) {
		incrBuildLvsTowardsNewNode(net,gn,step->domainFrozen,step->frozenLvCount);
	} else {
		incrBuildLvsTowardsNewNode(net,gn,NULL,NULL);
	}

	//step->profiling = measureTime(timer,step->profiling,"  other LVs");

	//timerFree(timer);

	return net;
}

/*
 * The given problem is extended by the given arc
 * especially the problems c-net and analysis are extended
 * and the domains are build anew
 *
 * the problem will be extended in place and will not be reusable for its old purpose!
 *
 * step is only given for profiling, the problem has to be bonded with the step somewhere else
 */
Problem extendProblem(Problem P, ChunkIncrement chunk, IncrementalStep step) {
	cdgPrintf(CDG_INFO, "extending Problem %s \n", P->id);
	cdgPrintf(CDG_INFO, "new Token(s) = %s \n", chunk->string);

#ifdef DEBUG_INCR_EXTEND_PROBLEM
	cdgPrintf(CDG_DEBUG, "lg width %d\n", P->lgWidth);
	cdgPrintf(CDG_DEBUG, "max tp %d\n", P->maxTimepoint);
	cdgPrintf(CDG_DEBUG, "slots %d\n", P->nrSlots);
	cdgPrintf(CDG_DEBUG, "lvs %d\n", P->nrLVs);
	cdgPrintf(CDG_DEBUG, "lexemes %d\n", P->nrLexemes);
#endif

	int i; List tmp;
	ConstraintNet net = P->net;
	Timer timer = timerNew();

	// freezing stuff
	if(incrFreezeLevel != FLTnone) {// reset frozen slots
		if(step->slotFrozen)
				bvDelete(step->slotFrozen);
		step->slotFrozen = bvNew(P->nrSlots + chunk->length * P->nrLevels); // nrslots is not increased here, yet
		bvSetAllElements(step->slotFrozen, FALSE);

		if(step->domainFrozen)
					bvDelete(step->domainFrozen);
		step->domainFrozen = bvNew(P->nrSlots + chunk->length * P->nrLevels); // nrslots is not increased here, yet
		bvSetAllElements(step->domainFrozen, FALSE);

		if(incrFreezeLevel == FLTsimulate) {
			// reset frozen marks
			int j;
			for(j=0;j<vectorSize(step->problem->net->values);j++) {
				LevelValue lval= vectorElement(step->problem->net->values,j);
				lval->isFrozen = FALSE;
			}
			bvSetAllElements(step->problem->lg->isFrozenNode,FALSE);
		}
	}
	if(incrFreezeLevel != FLTnone &&
			(!incrOnlyFreezeWhenGoalMet ||
					step->previousStep->score >= step->previousStep->goal))
	{
		List stableStructures;
		if(incrFreezingHeuristics == FHconflictFreeProjective) { // normales freezing
			stableStructures = searchStableSubstructures(step,
					INCR_SUBSTRUC_CONFLICTS |
					INCR_SUBSTRUC_LEAVES |
					INCR_SUBSTRUC_RIGHT_BRANCHES,
					inputCurrentGrammar->levels);
		} else if(incrFreezingHeuristics == FHradical) { // radical
			stableStructures = searchStableSubstructures(step,
					INCR_SUBSTRUC_NONSPEC,
					listPrependElement(listNew(),inputCurrentGrammar->levels->item));
		} else if(incrFreezingHeuristics == FHnotSoRadical) {
			stableStructures = searchStableSubstructures(step,
					INCR_SUBSTRUC_NONSPEC |
					INCR_SUBSTRUC_RIGHT_BRANCHES |
					INCR_SUBSTRUC_LEAVES,
					listPrependElement(listNew(),inputCurrentGrammar->levels->item));
		} else if(incrFreezingHeuristics == FHlookahead) {
			// frozen domains will be determined elsewhere by slotposition,
			// no need to check treestructure
			stableStructures = listNew();
		} else {
			stableStructures = listNew();
			cdgPrintf(CDG_INFO,"WARNING: Unimplemented freezing heuristics");
		}

		step->stableStructures= stableStructures;

		if(incrFreezingHeuristics == FHlookahead) {
			freezeByFixedLookaheadSize(step,incrLookaheadSize);
		} else {
			step->stableStructures= stableStructures;
			tmp= stableStructures;
			while(tmp) { IncrSubtree st = tmp->item; tmp=tmp->next;
				//int domain = lvIndex(st->root);
				//step->frozenLexemes = listAppendList(step->frozenLexemes,
				freezeLexemes(st,P); //);
				//step->frozenLVs = listAppendList(step->frozenLVs,
				freezeLVsInSubtree(st,P,step);//);
			}
		}
	}
	step->profiling = measureTime(timer,step->profiling," freezing");

	List l;
	int tp;
	if(useVirtualNodes) {
		tp= P->lg->ve->vStartID;
	} else {
		tp= P->lg->max;
	}
	for(l=chunk->words;l;l=l->next) {
		String nextWord = l->item;
		Arc arc = newArc(tp, tp + 1, tp, strRegister(nextWord), 0.0, NULL, 0, 0);
		tp++;

		// extending constraintnet, introducing new lexemnodes and new levelvalues
		net = extendNet(net,arc,step);
	}

	if(useVirtualNodes) {
		if(incrFreezeLevel == FLTnone) {
			cnManageVirtualNodes(net,P->best, NULL, NULL);
		} else {
			cnManageVirtualNodes(net,P->best,
							step->slotFrozen,
							step->frozenLvCount);
		}
	}

	net = postExtendNetCleanup(net,step);

	step->profiling = measureTime(timer,step->profiling," extend net total");

	if (P->domains) {
		//cdgPrintf(CDG_INFO, "  cleaning old domains \n");
		for (i = 0; i < P->nrSlots; i++)
			listDelete(P->domains[i]);
		memFree(P->domains);
		P->domains = NULL;
	}

	P->net = net;
	P->lg = net->lexemgraph;
	P->minTimepoint = net->lexemgraph->min;
	P->maxTimepoint = net->lexemgraph->max;
	P->nrTimepoints = net->lexemgraph->max - net->lexemgraph->min;
	P->nrSlots = P->maxTimepoint * P->nrLevels;
	P->nrActiveSlots = P->nrTimepoints * P->nrActiveLevels;
	P->killedByTimelimit=FALSE;

	// can't introduce pressure without making sure currently active elements will survive
	//Number minScore = 1.0;
	//for(i=0; i< vectorSize(P->best->slots);i++) {
	//	LevelValue lv =vectorElement(P->best->slots,i);
	//	minScore= min(minScore,lv->score);
	//}
	//P->globalMinimum= max(P->globalMinimum * incPressureRedux, globalPressure);

	P->globalMinimum=0.0;
	P->globalMaximum=1.0;

	P->patchAttempts =0;
	P->successfulPatches=0;

	//step->profiling = measureTime(timer,step->profiling," problem misc");

	// unremovable conflicts handling
	// only remove non-persistent conflicts, especially NONSPEC-related ones
	// cdgPrintf(CDG_INFO, "resetting unremovable conflicts list:\n");
	List newUnremovable = listNew();
	List temp = P->unremovable;
	if(!incrResetUnremovableConflicts) {
		while(temp) { // keep persistent conflicts
			ConstraintViolation cv = temp->item;
			temp=temp->next;
			String id= strRegister(cv->constraint->id);
			if(listContains(incrVolatileConstraints,cv->constraint)) {
				cdgPrintf(CDG_INFO, " removing Conflict %s from list\n", id);
			} else {
				cdgPrintf(CDG_INFO, " keeping Conflict %s on list \n", id);
				newUnremovable = listAppendElement(newUnremovable,cv);
			}
		}
	}
	listDelete(P->unremovable);
	P->unremovable = newUnremovable;

	//step->profiling = measureTime(timer,step->profiling," unremovable handling");

	buildLexemeTable(P); // P->Lexems is set here

	//step->profiling = measureTime(timer,step->profiling," build lexem table");
	step->profiling = measureTime(timer,step->profiling," problem misc");

	/* build domains */
#ifdef DEBUG_INCR_EXTEND_PROBLEM
	cdgPrintf(CDG_INFO, "Building domains...\n");
#endif

	P->domains = (List *) memMalloc(sizeof (List) * P->nrSlots);
	for (i = 0; i < P->nrSlots; i++) {
	  P->domains[i] = NULL;
	}
	P->nrLVs=0;
	P->nrUndeletedLVs=0;
	importLVs(P, 0);

	step->profiling = measureTime(timer,step->profiling," rebuilding domains");

#ifdef DEBUG_INCR_EXTEND_PROBLEM
	cdgPrintf(CDG_DEBUG, "new probl specs: \n");
	cdgPrintf(CDG_DEBUG, "lg width %d\n", P->lgWidth);
	cdgPrintf(CDG_DEBUG, "max tp %d\n", P->maxTimepoint);
	cdgPrintf(CDG_DEBUG, "slots %d\n", P->nrSlots);
	cdgPrintf(CDG_DEBUG, "lvs %d\n", P->nrLVs);
	cdgPrintf(CDG_DEBUG, "lexemes %d\n", P->nrLexemes);
#endif

	/* build Analysis from old one */
	Analysis newBest = extendAnalysis(P->best,P,chunk->length, step);
	P->best=newBest;
	P->current= aClone(newBest);
	P->globalMinimum = newBest->b->hard?0.0:newBest->b->soft;
	step->profiling = measureTime(timer,step->profiling," extending Analysis");

	if(incrOptimizeNet) {
		List l = NULL;
		for(i = 0; i < P->nrSlots; i++) {
			if(!slotActive(P->best,i)) {
				continue;
			}
			l = listPrependElement(l, (Pointer)i);
		}
		dCheckAll(P, l);
		step->profiling = measureTime(timer,step->profiling," optimizeNet");
	}

	rememberProblem(P);
	step->profiling = measureTime(timer,step->profiling," remember Problem");

	if(incrFreezeLevel != FLTnone) {
		// sammle eingefrorene LVals zu debuggingzwecken
		step->frozenLVs= listNew();
		for(i=0; i< vectorSize(P->net->values); i++) {
			LevelValue lv = vectorElement(P->net->values,i);
			if(lv->isFrozen)
				step->frozenLVs = listPrependElement(step->frozenLVs, lv);
		}

		// das selbe f�r Lexeme
		step->frozenLexemes= listNew();
		for(i=0; i< vectorSize(P->net->lexemgraph->nodes); i++) {
			LexemNode ln = vectorElement(P->net->lexemgraph->nodes,i);
			if(bvElement(P->net->lexemgraph->isFrozenNode, ln->no))
				step->frozenLexemes = listPrependElement(step->frozenLexemes, ln);
		}
		step->profiling = measureTime(timer,step->profiling," log frozen elements");
	}

	// fertig;
	cdgPrintf(CDG_INFO, "done extending problem\n");
	timerFree(timer);
	return P;
}



/********************************* printer ***************************************/

/*
 * Prints token, parse and time needed of an incremental step
 * if recursively is set, the previous steps leading to this one are printed, too
 */
void incrPrintStep(IncrementalStep step, Boolean recursively) {
	if(recursively && step->previousStep) {
		incrPrintStep(step->previousStep, TRUE);
		cdgPrintf(CDG_INFO, "---------------------------------------------\n");
	}

	cdgPrintf(CDG_INFO, "Incremental step for token: %s\n", step->word);
	cdgPrintf(CDG_INFO, "   total tokens: %d\n", step->iteration +1);
	cdgPrintf(CDG_INFO, "   time needed : %dms\n", step->timeNeeded);
	//cnPrint(CDG_INFO,step->problem->net);
	parsePrint2(step->parse, TRUE, frobIgnoreThreshold);
	cdgPrintf(CDG_INFO, "\n");
}

/**
 * prints all elements frozen in that step
 */
void incrPrintFrozenStuff(IncrementalStep step) {
	List tmp;
	int i;
	cdgPrintf(CDG_INFO,"Frozen LVs: %d\n", listSize(step->frozenLVs));
	/*for(tmp= step->frozenLVs;tmp; tmp= tmp->next) {
		LevelValue lv = tmp->item;
		lvPrint(CDG_INFO,lv,NULL,1);
		cdgPrintf(CDG_INFO,"\n");
	}*/
	cdgPrintf(CDG_INFO,"---------------------------------\n");
	cdgPrintf(CDG_INFO,"Frozen Lexemes: %d\n", listSize(step->frozenLexemes));
	for(tmp= step->frozenLexemes;tmp; tmp= tmp->next) {
		LexemNode ln = tmp->item;
		cdgPrintf(CDG_INFO,"%s\n", ln->lexem->description);
	}

	cdgPrintf(CDG_INFO,"Frozen domains:\n");
	for(i =0;i<bvSize(step->domainFrozen);i++) {
		if(bvElement(step->domainFrozen,i))
			cdgPrintf(CDG_INFO,"%d, ", i);
	}
	cdgPrintf(CDG_INFO,"\n");
}

/*
 * Prints a listing of the seconds needed for each token of the incremental process
 * returns total time needed
 */
unsigned long incrPrintTimeNeeded(IncrementalStep step, Boolean printHeader) {

	if(printHeader) {
		cdgPrintf(CDG_INFO, "+====================================================================+\n");
		cdgPrintf(CDG_INFO, "|  id -   time needed -  repairs - score -  vn - word                |\n");
		cdgPrintf(CDG_INFO, (incrFrobbingGoalDecline==0.0)?
							"|     -   (!:t-limit) - succ/all -       - !=m -                     |\n":
							"|     -   (!:t-limit) - succ/all - /goal - !=m -                     |\n");
		cdgPrintf(CDG_INFO, "+--------------------------------------------------------------------+\n");
	}

	unsigned long ret= step->timeNeeded;
	if(step->previousStep) {
		ret += incrPrintTimeNeeded(step->previousStep, FALSE);
	}
	char buffer[1024];
	if(incrFrobbingGoalDecline==0.0) {
		sprintf(buffer, "%4.3f", step->score);
	} else {
		sprintf(buffer, "%4.3f/ %4.3f", step->score, step->goal);
	}

	cdgPrintf(CDG_INFO, "| %3d - %8.3f %s -  %3d/%3d - %s - %s - %s\n",
			step->iteration,
			step->timeNeeded/1000.0, step->killedByTimeLimit?"sec!":"sec ",
			step->successfulPatches, step->patchAttempts,
			buffer,
			step->virtualNodeMatch ? " m!" : "   ",
			step->word);

	if(printHeader) {
		cdgPrintf(CDG_INFO, "+====================================================================+\n");
	}

	return ret;
}

/*
 * Prints a lot of details about the given step and it's predecessors
 * used mainly for debugging
 */
void incrPrintDetails(IncrementalStep step, int flags) {
	if(step->previousStep) {
		incrPrintDetails(step->previousStep, flags);
	}
	List tmp;
	List tmp2;
	unsigned long totalFrobbing=0;
	unsigned long totalExtend=0;

	RepairAttempt ra;
	cdgPrintf(CDG_INFO, "================================================\n");
	cdgPrintf(CDG_INFO, "%3d - %8.3fsec - %1.3f - %s\n",
				step->iteration,
				step->timeNeeded/1000.0,
				step->score,
				step->word);
	if(step->score < step->goal)
		cdgPrintf(CDG_INFO,"Goal not met!\n");
	if(step->reanalysed)
		cdgPrintf(CDG_INFO,"Reanalysed!\n");
	cdgPrintf(CDG_INFO, "Patches: %d/%d - steps: %d - unEvals: %li - binEvals: %li - lvals: %d - lexemes : %d\n",
					step->successfulPatches, step->patchAttempts,
					step->frobbingSteps,
					step->evalUnary,
					step->evalBinary,
					step->lvalsTotal,
					step->lexemesTotal);
	cdgPrintf(CDG_INFO, "LVs built: total %d, deleted %d, postponed %d, in_net %d \n",
			step->lvCountTotal,
			step->lvCountDeleted,
			step->lvCountPostponed,
			step->lvCountBuild);

	cdgPrintf(CDG_INFO, "------------------------------------------------\n");
	if(flags & INCR_WRITE_TIME) {
		tmp = step->profiling;
		while(tmp) {
			ProfilingMeasurePoint mp= tmp->item;
			tmp= tmp->next;
			cdgPrintf(CDG_INFO, "%-20s: %8.3fsec\n", mp->name,mp->time/1000.0);
			if(strcmp(mp->name,"frobbing")==0) {
				totalFrobbing+=mp->time;
			} else if(strcmp(mp->name,"extend problem total")==0) {
				totalExtend+=mp->time;
			}
		}
		cdgPrintf(CDG_INFO, "------------------------------------------------\n");
	}
	if(flags & INCR_WRITE_UNREM) {
			tmp = step->unremovedConflicts;
			cdgPrintf(CDG_INFO, "unremovable conflicts\n");
			while(tmp) {
				cdgPrintf(CDG_INFO,"%s\n",tmp->item);
				tmp= tmp->next;
			}
			cdgPrintf(CDG_INFO, "------------------------------------------------\n");
		}
	if(flags & INCR_WRITE_REPAIRATTEMPTS) {
		tmp = step->repairAttempts;
		while(tmp) {
			ra= tmp->item;
			tmp= tmp->next;
			tmp2= ra->otherConflicts;
			cdgPrintf(CDG_INFO, " %s(%8.3fsec  - %s : ",
					ra->successful?"+":"-",
					ra->time/1000.0,
					ra->primaryConflict->id);
			while(tmp2) {
				cdgPrintf(CDG_INFO, "%s, ", ((Constraint)tmp2->item)->id);
				tmp2=tmp2->next;
			}
			cdgPrintf(CDG_INFO, ")\n");
		}
		cdgPrintf(CDG_INFO, "------------------------------------------------\n");
	}
	if(flags & INCR_WRITE_REAPPEARING_LVALS) {
		cdgPrintf(CDG_INFO,"reappeared LVs: \n");
		tmp= step->returnedLVs;
		while(tmp) { LevelValue lv = tmp->item; tmp=tmp->next;
			lvPrint(CDG_INFO,lv,NULL,0);
			cdgPrintf(CDG_INFO,"\n");
		}
	}
	if(flags & INCR_WRITE_REAPPEARING_LEXEMES) {
		cdgPrintf(CDG_INFO,"reappeared Lexemes: \n");
		tmp= step->returnedLexemes;
		while(tmp) { LexemNode ln = tmp->item; tmp=tmp->next;
			cdgPrintf(CDG_INFO,"%s\n",ln->lexem->description);
		}
	}
	if(flags & INCR_WRITE_STABLE_SUBTREES) {
		cdgPrintf(CDG_INFO,"stable substructures: \n");
		tmp= step->stableStructures;
		while(tmp) { IncrSubtree subtree = tmp->item; tmp=tmp->next;
			lvPrint(CDG_INFO, subtree->root, NULL, 1);
			cdgPrintf(CDG_INFO,"\n");
		}
		cdgPrintf(CDG_INFO,"frozen LVs/Lexemes: %d/ %d \n", listSize(step->frozenLVs), listSize(step->frozenLexemes));
		cdgPrintf(CDG_INFO,"LV freezing occasion: %d; %d; %d; %d \n",vectorElement(step->frozenLvCount,0),vectorElement(step->frozenLvCount,1),vectorElement(step->frozenLvCount,2),vectorElement(step->frozenLvCount,3));
		cdgPrintf(CDG_INFO,"unfrozen LVs: \n");
		tmp= step->unfrozenLVs;
		while(tmp) { LevelValue lv = tmp->item; tmp=tmp->next;
			lvPrint(CDG_INFO,lv,NULL,0);
			cdgPrintf(CDG_INFO,"\n");
		}
		cdgPrintf(CDG_INFO,"unfrozen Lexemes: \n");
		tmp= step->unfrozenLexemes;
		while(tmp) { LexemNode ln = tmp->item; tmp=tmp->next;
			cdgPrintf(CDG_INFO,"%s\n",ln->lexem->description);
		}
	}
}

/*
 * Writes the resulting parse of the given incremental step as annotation to a file
 * named {given filename} + ".cda"
 * if recursive is set, every step in the incremental process leading to the given step is
 * written, too
 */
void incrWriteAnnotationToFile(IncrementalStep step, Boolean recursive, String name) {
	if(recursive && step->previousStep) {
		incrWriteAnnotationToFile(step->previousStep,TRUE,name);
	}
	AnnoEntry ae = parse2annotation(step->parse);
	String filename= strPrintf("%s-%d.cda",name,step->iteration);

	cdgPrintf(CDG_INFO, "writing %s ... \n", filename);
	writeAnnotation(ae,filename);
	freeAnnoEntry(ae);
}


/*
 * recursive helper for incrWriteProfilingToFile
 * see there for further information
 *
 */
void writeProfilingToFileHelper(IncrementalStep step, FILE *file, int flags) {
	if(step->previousStep)
		writeProfilingToFileHelper(step->previousStep, file,flags);

	if(flags& INCR_WRITE_TOKEN) {
		fprintf(file, "token: %d - %s\n",step->iteration, step->word);
		fprintf(file, "nrtokens: %d\n",step->nrTokens);
	}
	if(flags& INCR_WRITE_TIME) {
		fprintf(file, "time: %lu\n",step->timeNeeded);
		if(step->killedByTimeLimit) {
			fprintf(file, "timelimit: yes\n");
		}
	}
	if(flags& INCR_WRITE_SCORE)
		fprintf(file, "score: %5.3f\n",step->score);
	if(flags& INCR_WRITE_PATCHATTEMPTS)
		fprintf(file, "steps: %d\n", step->frobbingSteps);
	if(flags& INCR_WRITE_PATCHATTEMPTS)
		fprintf(file, "patches, success / total: %d / %d \n", step->successfulPatches, step->patchAttempts);
	if(flags& INCR_WRITE_EVALUATION_COUNT)
		fprintf(file, "evals, unary / binary: %li / %li \n", step->evalUnary, step->evalBinary);
	if(flags& INCR_WRITE_AMOUNTS)
		fprintf(file, "count, LV / Lex: %d - %d \n", step->lvalsTotal, step->lexemesTotal);
	if(flags& INCR_WRITE_UNREM) {
		fprintf(file, "unremoved:");
		List tmp = step->unremovedConflicts;
		while(tmp) {
			fprintf(file, tmp->item);
			tmp= tmp->next;
			if(tmp)
				fprintf(file,",");
		}
		fprintf(file,"\n");
	}
	if(flags & INCR_WRITE_REAPPEARING_LEXEMES) {
		fprintf(file, "reappearing_lexemes:");
		List tmp = step->returnedLexemes;
		while(tmp) {
			LexemNode ln = tmp->item;
			tmp= tmp->next;
			fprintf(file, "%s",ln->lexem->description);
			if(tmp)
				fprintf(file,",");
		}
		fprintf(file,"\n");
	}
	if(flags & INCR_WRITE_REAPPEARING_LVALS) {
		fprintf(file, "reappearing_lvs:");
		List tmp = step->returnedLVs;
		while(tmp) {
			LevelValue lv = tmp->item;
			tmp= tmp->next;
			fprintf(file, "%s",lv->label);
			if(tmp)
				fprintf(file,",");
		}
		fprintf(file,"\n");
	}
	if(flags & INCR_WRITE_STABLE_SUBTREES && incrFreezeLevel != FLTnone) {
		fprintf(file,"frozen LVs: %d \n",     listSize(step->frozenLVs));
		fprintf(file,"frozen Lexemes: %d \n", listSize(step->frozenLexemes));

		fprintf(file,"unfrozen LVs:"); //     listsize(step->unfrozenLVs));
		List tmp = step->unfrozenLVs;
		while(tmp) { LevelValue lv= tmp->item; tmp= tmp->next;
			fprintf(file,lv->label);
			if(tmp)
				fprintf(file,",");
		} fprintf(file,"\n");
		fprintf(file,"unfrozen Lexemes:");
		tmp = step->unfrozenLexemes;
		while(tmp) { LexemNode ln= tmp->item; tmp= tmp->next;
			fprintf(file,ln->lexem->description);
			if(tmp)
				fprintf(file,",");
		} fprintf(file,"\n");
	}
	if(flags & INCR_WRITE_REPAIRATTEMPTS) {
		List tmp = step->repairAttempts;
		fprintf(file,"repairattempts:\n");
		while(tmp) {
			RepairAttempt ra= tmp->item;
			tmp= tmp->next;
			fprintf(file, "%s : %3.3f : %d : %s\n",
					ra->successful?"y":"n",
					ra->time/1000.0,
					listSize(ra->otherConflicts)+1,
					ra->primaryConflict->id);
		}
		fprintf(file,";");
	}
	fprintf(file, ";\n\n");
}

/*
 * Writes some profiling information about the step and its predecessors
 * to a file with the given filename
 *
 * information written can be constrolled via flags:
 * per step:
 * INCR_WRITE_TOKEN         : token string and position
 * INCR_WRITE_TIME          : time needed for the step
 * INCR_WRITE_SCORE         : final score of the analysis
 * INCR_WRITE_UNREM         : unremovable conflicts in this step
 * INCR_WRITE_PATCHATTEMPTS : how many patches where tried, how many succeded
 *
 * once:
 * INCR_WRITE_REMAINING     : conflicts still present in the final analysis
 *
 */
void incrWriteProfilingToFile(IncrementalStep step, String name, int flags) {
	String filename= strPrintf("%s.cdp",name);
	cdgPrintf(CDG_INFO, "writing %s ... \n", filename);
	FILE *file = fopen(filename, "w");

	if (!file) {
		cdgPrintf(CDG_WARNING, "WARNING: cannot write to file %s!\n", name);
	}

	writeProfilingToFileHelper(step, file, flags);

	if(flags &  INCR_WRITE_REMAINING) {
		fprintf(file,"REMAINING:");
		List tmp = step->problem->unremovable;
		while(tmp){
			String s = cvToString((ConstraintViolation)tmp->item,TRUE);
			tmp= tmp->next;
			fprintf(file,"%s", s);
			if(tmp)
				fprintf(file,",");
		}
		fprintf(file,"\n");
	}

	fprintf(file, ";;\n\n");
	fclose(file);
}




/*
void incrWriteStepToFile(IncrementalStep step, Boolean recursive, String name) {
	FILE *file;
	String filename= strPrintf("%s.cda",name);

	if(recursive && step->previousStep) {
		incrWriteStepToFile(step->previousStep,TRUE,name);
		file = fopen(filename, "a");
	} else {
		file = fopen(filename, "w");
	}

	if(!file) {
		cdgPrintf(CDG_WARNING, "WARNING: can't write to %s!\n", filename);
		return;
	}

	// title
	fprintf(file, "\n%s : %s <->",
			formatIdentifier(annoname),
			formatIdentifier(ae->lattice));
} */


/*********** Incremental ******************/

/*
 * given a previous step and a new word,
 * this function extends the problem from the previous step with the given word
 * and frobbs it
 *
 * final says, if this iteration should be treated as the final iteration over the sentence
 *
 * result is the next incremental step
 *
 */
IncrementalStep incrNextStep(IncrementalStep lastStep, ChunkIncrement nextChunk, Boolean final, List lookahead) {
	cdgPrintf(CDG_INFO, "==============================================\n");

	int i;
	Timer timer = timerNew(); // total
	Timer timer2 = timerNew(); // intermediate
	if (cdgTimeLimit != 0) {
		timerSetAlarm(cdgTimeLimit);
	}
	IncrementalStep step = incrStepNew();
	step->problem = lastStep->problem;
	step->previousStep = lastStep;
	step->word = nextChunk->string;
	step->mode="fully incremental";
	step->iteration = lastStep->iteration+1;
	step->profiling = listNew();
	step->deletedLVs = lastStep->deletedLVs;
	step->returnedLVs= listNew(); //lastStep->returnedLVs;
	step->deletedLexemes= lastStep->deletedLexemes;
	step->returnedLexemes= listNew();
	step->unremovedEstablishedConflicts = lastStep->unremovedEstablishedConflicts;
	step->nrTokens = lastStep->nrTokens + nextChunk->length;
	if(incrTaggerFeedback) {
		step->selectedTags= aGetSelectedCats(lastStep->problem->best);
	}

	if(final) { // if this iteration is the final one, different rules apply:
		step->goal= 0.0; // no goal must be set as we can't postpone further
		prepareFinalStep(lastStep->problem); // penalties of constraints with special scores for partial sentences are switched to the value for complete sentences
		step->problem->lg->complete = TRUE;  // the lexemgraph is marked as complete
	} else if(incrFrobbingGoalDecline > 0.0) { // if this is not the last step, determine new goal
		if(lastStep->score < lastStep->goal) {
			step->goal= lastStep->goal * incrFrobbingGoalDecline;
			cdgPrintf(CDG_INFO, "last step's goal was not met, reducing goal \n");
		} else {
			step->goal= lastStep->score * incrFrobbingGoalDecline;
		}
		cdgPrintf(CDG_INFO,"new goal: %.3f \n", step->goal);
	}

	step->profiling = measureTime(timer2,step->profiling,"initialize step");

	if(lookahead) {
		step->problem->lg->lattice->lookahead = lookahead;
	}

	step->problem = extendProblem(lastStep->problem,nextChunk, step);
	step->profiling = measureTime(timer2,step->profiling,"extend problem total");

	cdgPrintf(CDG_INFO, "Frobbing iteration %d \n", step->iteration);
	Problem P= step->problem;
	launchFrobbing(P, step->goal);

	step->profiling = measureTime(timer2,step->profiling,"frobbing");

	if(incrRetryUnfrozen && incrFreezeLevel == FLTfull && resultTooBad(P,step->previousStep->score)) {
		cdgPrintf(CDG_INFO,"reinterpretatiion, current result is too bad (%.3f ,was %.3f), unfreezing...\n",P->best->b->soft,step->previousStep->score);
		step->reanalysed = TRUE;

		unfreezeProblem(P);
		// TODO prevent retagging or redo it with feedback

		if (cdgTimeLimit != 0) {
			timerSetAlarm(cdgTimeLimit);
		}
		launchFrobbing(P,step->goal);

		step->profiling = measureTime(timer2,step->profiling, "reanalysis");
	}

	cdgPrintf(CDG_INFO, "iteration done \n");

	if(incrFreezeLevel ==FLTsimulate || (incrFreezeLevel== FLTfull && incrRetryUnfrozen)) {
		// check if deleted or frozen lvs are reused later on
		Vector selected= step->problem->best->slots;
		for(i=0;i<vectorSize(selected);i++) {
			LevelValue lv = vectorElement(selected,i);

			if(!lv)
				continue;

			if(listContains(step->deletedLVs,lv)) {
				step->returnedLVs = listAppendElement(step->returnedLVs, lv);
			}
			if(listContains(step->frozenLVs,lv)) {
				step->unfrozenLVs = listAppendElement(step->unfrozenLVs, lv);
			}
		}

		// check if deleted or frozen lexemes are reused later on
		ByteVector unselected= step->problem->best->isUnselectedLexeme;
		for(i=0;i<bvSize(unselected);i++) {
			if(!bvElement(unselected,i)) {
				LexemNode ln = vectorElement(step->problem->lg->nodes,i);
				if(listContains(step->deletedLexemes,ln)) {
					step->returnedLexemes = listAppendElement(step->returnedLexemes, ln);
				}
				if(listContains(step->frozenLexemes,ln)) {
					step->unfrozenLexemes = listAppendElement(step->unfrozenLexemes, ln);
				}
			}
		}
		step->profiling = measureTime(timer2,step->profiling,"unfreezing check");
	}

	step->timeNeeded=timerElapsed(timer);
	getInfoFromProblem(step,step->problem);

	for(i =0; i< vectorSize(step->problem->best->slots);i++) {
		LevelValue lv = vectorElement(step->problem->best->slots,i);

		if(!lv)
			continue;

		if(lv->wasReactivated) {
			cdgPrintf(CDG_INFO,"Reactived LV in result: ");
			lvPrint(CDG_INFO,lv,NULL,1);
			cdgPrintf(CDG_INFO,"\n");
		}
	}

	if(final) {
		cdgPrintf(CDG_INFO,"Conflict persistency:\n");
		List l = step->unremovedEstablishedConflicts;
		for(;l;l=l->next) {
			String confl = l->item;
			if(listContains(step->unremovedConflicts,confl)) {
				// an established conflict proved to be persistent
				cdgPrintf(CDG_INFO," persistent:%s\n",confl);
			} else {
				// an established conflict was found to be fluctuating
				cdgPrintf(CDG_INFO," fluctuating:%s\n",confl);
			}
		}

		postpareFinalStep();

	}

	step->profiling = measureTime(timer2,step->profiling,"finalizing(generating parse)");

	timerFree(timer);
	timerFree(timer2);

	if (cdgTimeLimit != 0) {
		timerStopAlarm();
	}

	return step;
}

/*
 * this function generates an initial incremental step (i.e. with no previous step)
 * and frobbs the resulting (very small) problem
 */
IncrementalStep incrInitialStep(ChunkIncrement chunk, Boolean complete, List lookahead) {
	//cdgPrintf(CDG_INFO,"Initiating incremental parsing\n");
	double pressure = globalPressure;//0.0;
	int counter=0;
	int i;
	List l;
	Lattice lat =NULL;
	ConstraintNet cn = NULL;
	Problem P = NULL;
	Arc arc = NULL;
	String word;
	IncrementalStep step = incrStepNew();
	step->goal=incrFrobbingGoalDecline;
	step->profiling = listNew();
	step->nrTokens= chunk->length;

	Timer timer  =timerNew(); // total
	Timer timer2 =timerNew(); // intern
	if (cdgTimeLimit != 0) {
		timerSetAlarm(cdgTimeLimit);
	}

	cdgPrintf(CDG_INFO,"Building initial lattice\n");
	lat = (Lattice) memMalloc(sizeof (LatticeStruct));
	lat->id = strPrintf("wordgraph%d", counter++);
	lat->filename = strRegister("*INKR*");
	lat->lineNo = 0;
	lat->arcs = listNew();
	lat->lookahead = lookahead;

	for(l= chunk->words, i=0; l ;l= l->next,i++) {
		word = l->item;
		arc = newArc(i, i+1, i, strRegister(word), 0.0, NULL, 0, 0);
		lat->arcs = listAppendElement(lat->arcs, arc);
	}

	inputScanLattice(lat);

	step->profiling = measureTime(timer2,step->profiling,"initial lattice");

	cdgPrintf(CDG_INFO,"Building initial CNet\n");
	cn = cnBuild(lat,TRUE, complete);

	if(!cn) {
		cdgPrintf(CDG_ERROR,"ERROR: Constraintnet could not be built, possibly because of missing or flawed grammar!\n");
		return NULL;
	}

	if(useVirtualNodes) {
		cnInitVirtualNodes(cn,
				inputCurrentGrammar->vnspecs,
				step->slotFrozen,
				step->frozenLvCount);
	}

	step->profiling = measureTime(timer2,step->profiling,"initial cnet");
	cdgPrintf(CDG_INFO,"Building initial Problem\n");

	P = net2problem(cn, pressure, NULL);

	for(i=0; i<vectorSize(P->net->nodes);i++) {
		ConstraintNode cnode= vectorElement(P->net->nodes,i);
		incrUpdateNumberCaches(cnode);
	}
	aJudge(P->current);

	step->profiling = measureTime(timer2,step->profiling,"initial Problem");
	cdgPrintf(CDG_INFO,"Solving initial Problem\n");

	launchFrobbing(P,step->goal);

	step->profiling = measureTime(timer2,step->profiling,"Solving initial Problem");

	step->problem= P;
	step->iteration=0;
	step->previousStep=NULL;
	step->word= chunk->string;
	step->mode="full incremental";
	step->deletedLVs= listNew();
	step->returnedLVs= listNew();
	step->deletedLexemes= listNew();
	step->returnedLexemes= listNew();

	getInfoFromProblem(step,P);

	step->timeNeeded=timerElapsed(timer);
	step->profiling = measureTime(timer2,step->profiling,"finalizing");
	timerFree(timer);
	timerFree(timer2);

	return step;
}



/*
 * Parses a sentence via frobbing in a series of incremental steps,
 * beginning with a sentence just containing the first word
 * and ending with the whole sentence.
 * Several results from the previous steps are used to initialize the next problem
 *
 * there are two modes: automatic and interactive
 *
 * In automatic mode a lattice is given from the beginning
 * In interactive mode the program prompts for a new token after each step
 *
 * interactive mode is chosen if no Lattice is given
 */
IncrementalStep incrementalParsing(Lattice lat, Boolean finalize, Boolean onechunk) {
	Boolean interactive;
	String input;
	ChunkIncrement increment;
	IncrementalStep curStep;
	List arcs = NULL;
	List lookahead = NULL;
	Boolean complete = FALSE;
	Vector v;
	int i;

	if(lat==NULL) {
		interactive= TRUE;
	} else {
		interactive= FALSE;
	}

	suppressPartialSolutions = TRUE; // incremental parsing writes out parses itself

	// initial step
	if(interactive) {
		input= promptForNextWord("next","-q for quit");
		increment= chunkIncrementNew(input,TRUE);
		// TODO, some means to mark this first increment as complete
	} else {
		arcs= lat->arcs;
		if(onechunk) {
			v = vectorNew(listSize(arcs));

			while(arcs) {
				vectorAddElement(v, ((Arc)arcs->item)->word);
				arcs = arcs->next;
			}

			increment = chunkIncrementNew2(v);
			// TODO, some means to mark this first chunk as complete
		} else {
			input= ((Arc)arcs->item)->word;
			arcs= arcs->next;
			increment = chunkIncrementNew(input,FALSE); // only one word
			complete = incrAutoFinalize?(arcs?FALSE:TRUE):FALSE; // if arcs are left, the sentence is not complete

			if(incrTaggerLookaheadSize >0) {
				lookahead = listNew();
				for(i=1; i<= incrTaggerLookaheadSize && i < listSize(arcs); i++) {
					lookahead = listAppendElement(lookahead, listNthElement(arcs,i));
				}
			}
		}
	}
	curStep = incrInitialStep(increment, complete, lookahead);

	if(!curStep) {
		cdgPrintf(CDG_ERROR,"ERROR: Building initial incremental step was unsuccessful, aborting incremental parsing!\n");
		return NULL;
	}

	// loop
	Boolean finished= FALSE;
	if(!arcs && !interactive)
		finished = TRUE; // catch, if there is only one token in the input lattice
	while(!finished && !cdgCtrlCTrapped) {
		if(interactive) {
			incrPrintStep(curStep,FALSE);
			input= promptForNextWord("next","-q for quit");
			if(!strcmp(input,"-q")) {
				// finish immediately
				finished=TRUE;
				// TODO finalize?
				break;
			}
			increment = chunkIncrementNew(input,TRUE);
		} else {
			input= ((Arc)arcs->item)->word;
			arcs= arcs->next;
			if(!arcs) {
				finished = TRUE; // finish after this step
			}
			if(incrTaggerLookaheadSize >0) {
				lookahead = listNew();
				for(i=1; i<= incrTaggerLookaheadSize && i < listSize(arcs); i++) {
					lookahead = listAppendElement(lookahead, listNthElement(arcs,i));
				}
			}
			increment = chunkIncrementNew(input,FALSE);
		}

		curStep= incrNextStep(curStep,increment,finished && finalize, lookahead);
	}

	incrPrintStep(curStep, FALSE);

	suppressPartialSolutions = FALSE;

	return curStep;
}

/*
 * Continues an incremetal process, represented by the given step,
 * if a string is given as next word, that one is used,
 * if word == NULL, the user is prompted for the next word
 */
IncrementalStep incrContinueStep(IncrementalStep step, Vector input, Boolean final) {
	IncrementalStep nextStep;
	if(input) {
		ChunkIncrement chunk = chunkIncrementNew2(input);
		nextStep= incrNextStep(step,chunk, final, NULL);
	} else {
		ChunkIncrement chunk = chunkIncrementNew(
				promptForNextWord("next"," "),
				TRUE);
		nextStep= incrNextStep(step,chunk, final, NULL);
	}
	incrPrintStep(nextStep,FALSE);
	return nextStep;
}


/**************** pseudo incremental stuff ****************************/

/*
 * calls combined once for the whole sentence,
 * but the result is wrapped by a IncrementalStepStruct,
 * for easy comparison with incremental methods
 */
IncrementalStep incrNonIncremental(Lattice wholeLat, int frobMethod) {
			Timer timer = timerNew();
			if (cdgTimeLimit != 0) {
				timerSetAlarm(cdgTimeLimit);
			}

			ConstraintNet cn = cnBuild(wholeLat,TRUE, TRUE);
			if(!cn) {
				cdgPrintf(CDG_ERROR,"ERROR: Constraintnet could not be built, possibly because of missing or flawed grammar!\n");
				return NULL;
			}
			Problem P = net2problem(cn, 0.0, NULL);
			aJudge(P->current);
			double time1 = timerElapsed(timer)/ 1000.0;
			timerStart(timer);

			suppressPartialSolutions= TRUE; // incremental parsing writes out parses itself

			if(frobMethod==1) {
				dynamic(P);
			} else if(frobMethod==2) {
				threefold(P);
			} else {
				combined(P);
			}

			suppressPartialSolutions= FALSE;

			double time2 = timerElapsed(timer)/ 1000.0;

			IncrementalStep step= incrStepNew();
			step->problem= P;
			step->nrTokens= P->nrSlots;
			getInfoFromProblem(step,P);

			cdgPrintf(CDG_INFO, "contrast: time needed to build whole Problem at once\n %8.3f\n", time1);
			cdgPrintf(CDG_INFO, "contrast: time needed to frob whole Problem at once\n %8.3f\n", time2);
			timerFree(timer);

			return step;
	}

/*
 * This incremental parsing method builds a new net each time,
 * but initializes the analysis with the parse of the last step
 */
IncrementalStep incrPseudoIncrementalParsing(Lattice lat) {
	IncrementalStep lastStep= NULL;
	IncrementalStep curStep = NULL;
	Problem P;
	String word;
	List words = listNew();
	Parse parse= NULL;
	List arcs= lat->arcs;
	Timer timer = timerNew(); // total
	Timer timer2 = timerNew(); // intermediate

	suppressPartialSolutions = TRUE; // incremental parsing writes out parses itself

	while(arcs) {
		cdgPrintf(CDG_INFO, "======================================== \n");
		curStep= incrStepNew();

		timerStart(timer);
		timerStart(timer2);
		if (cdgTimeLimit != 0) {
			timerSetAlarm(cdgTimeLimit);
		}

		word= ((Arc)arcs->item)->word;
		arcs= arcs->next;
		cdgPrintf(CDG_INFO, "next word: %s \n", word);
		words=listAppendElement(words,word);

		Lattice curLat =makeLattice(words,strRegister("*pseudoincremental*"));
		ConstraintNet cnet= cnBuild(curLat, TRUE, arcs?FALSE:TRUE); // if arcs are left, the sentence is not complete
		if(!cnet) {
			cdgPrintf(CDG_ERROR,"ERROR: Constraint net could not be built, possibly because of missing or flawed grammar!\n");
			return NULL;
		}

		cdgPrintf(CDG_INFO, "building problem\n");
		P = net2problem(cnet,0.0,parse);
		curStep->nrTokens = P->nrSlots;

		if(!arcs && cnUseNonSpec) { // final step, no nonspec here
			prepareFinalStep(P);
			aJudge(P->best);
			aJudge(P->current);
			// TODO mark the lg as complete here and as partial else (currently frobbing assumes completeness)
		}

		curStep->profiling = measureTime(timer2,curStep->profiling,"build Problem");

		launchFrobbing(P,0.0);

		curStep->profiling = measureTime(timer2,curStep->profiling,"frob Problem");

		if(!arcs && cnUseNonSpec) {
			postpareFinalStep();
		}


		curStep->previousStep= lastStep;
		curStep->word = word;
		curStep->mode="pseudo incremental parsing";
		getInfoFromProblem(curStep,P);
		parse=curStep->parse;
		curStep->timeNeeded = timerElapsed(timer);

		lastStep=curStep;
	}
	timerFree(timer);

	suppressPartialSolutions = FALSE;

	return curStep;
}


/*
 * one iteration of prefix parsing,
 * the given step is used only to attach it to the new step structure,
 * no information of the old step is used to guide the next one
 */
IncrementalStep incrPrefixParsingStep(IncrementalStep lastStep, String newWord, Lattice lat, Boolean complete) {
	Timer timer = timerNew(); // total
	Timer timer2 = timerNew(); // intermediate
	if (cdgTimeLimit != 0) {
		timerSetAlarm(cdgTimeLimit);
	}

	IncrementalStep step = incrStepNew();
	if(lastStep) {
		step->previousStep = lastStep;
		step->iteration = lastStep->iteration+1;
		step->nrTokens= step->iteration;
	} else {
		step->previousStep = NULL;
		step->iteration = 0;
	}

	step->word = newWord;
	step->mode="prefix parsing";
	step->profiling = listNew();

	step->profiling = measureTime(timer2,step->profiling,"initialize step");

	cdgPrintf(CDG_INFO, "building constraint net\n");
	ConstraintNet cnet= cnBuild(lat, TRUE, complete);
	if(!cnet) {
		cdgPrintf(CDG_ERROR,"ERROR: Constraint net could not be built, possibly because of missing or flawed grammar!\n");
		return NULL;
	}
	cdgPrintf(CDG_INFO, "building problem\n");
	step->problem = net2problem(cnet,0.0,NULL);

	step->profiling = measureTime(timer2,step->profiling,"build problem");

	cdgPrintf(CDG_INFO, "Frobbing iteration %d \n", step->iteration);
	launchFrobbing(step->problem,0.0);
	cdgPrintf(CDG_INFO, "iteration done \n");

	step->profiling = measureTime(timer2,step->profiling,"frobbing");

	getInfoFromProblem(step,step->problem);

	step->timeNeeded=timerElapsed(timer);
	step->profiling = measureTime(timer2,step->profiling,"finalizing");

	timerFree(timer);
	timerFree(timer2);

	if (cdgTimeLimit != 0) {
		timerStopAlarm();
	}

	return step;
}


/*
 * parses each prefix of the given lattice independently
 * using the combined frobbing method,
 * but it builds a structure of incremental steps to manage the results
 * only useful to compare the results with fully incremental parsing results
 */
IncrementalStep incrPrefixParsing(Lattice lat) {
	String word;
	IncrementalStep curStep =NULL;
	List words = listNew();

	List arcs= lat->arcs;

	suppressPartialSolutions = TRUE; // incremental parsing writes out parses itself

	while(arcs) {
		cdgPrintf(CDG_INFO, "======================================== \n");
		word= ((Arc)arcs->item)->word;
		arcs= arcs->next;

		words=listAppendElement(words,word);
		cdgPrintf(CDG_INFO, "next word: %s \n", word);

		Lattice curLat = makeLattice(words, strRegister("*prefixparsing*"));

		if(!arcs && cnUseNonSpec) { // final step, don't use nonspec
			cnUseNonSpec = FALSE;
			curStep= incrPrefixParsingStep(curStep, word, curLat, TRUE);
			cnUseNonSpec = TRUE;
		} else {
			curStep= incrPrefixParsingStep(curStep, word, curLat, FALSE);
		}

	}

	suppressPartialSolutions = FALSE;

	incrPrintStep(curStep, FALSE);

	return curStep;
}


/* ---------------------------------------------------------------------- */
/* -- ENDOFFILE --------------------------------------------------------- */
/** @} */
