/*
 * Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
 *
 * This file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. For
 * more details read COPYING in the root of this distribution.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

/* -------------------------------------------------------------------------
 * @addtogroup LevelValue
 * @author Ingo Schroeder (see also AUTHORS and THANKS for more)
 * @date 24/4/97
 *
 *
 * @{
 */

#ifndef LEVELVALUE_H
#define LEVELVALUE_H

/* ---------------------------------------------------------------------- */
#include <blah.h>
#include "input.h"
#include "lexemgraph.h"

/* ---------------------------------------------------------------------- */

#ifdef SWIG
typedef LevelValueStruct *LevelValue;
#else
typedef struct {
  int no;                    /* id used to find scores in cache */
  List dependents;           /* modifying nodes */
  Level level;               /* level of modification relation */
  String label;              /* label of modification relation */
  List regents;              /* modified nodes */

  double score;              /* score */
  double nonFluctuatingScore;	     /* part of the score resulting from fluctuating constraints, used in incremental environment */
  List constraints;          /* list of violated constraints */
  double limit;              /* maximal attainable score of solution */
  Boolean isDeleted;         /* flags to indicate deletion */
  Boolean isFrozen;	// used for incremental Parsing, frozen LVs are not used but may be reactivated later on, isdeleted is not persistent between steps
  Boolean wasReactivated;
  int indexWRTNet;
} LevelValueStruct;
typedef LevelValueStruct *LevelValue;
#endif

/* ---------------------------------------------------------------------- */
#ifndef SWIG
#define NONSPEC ((Pointer)(-1))
//#define NNONSPEC ((Pointer)(-2))
#define root(a) (((int)a) == 0)
#define spec(a) (!(((int)a) == 0) && !(((int)a) == -1))
#define nonspec(a) (((int)(a)) == -1)
//#define nnonspec(a) (((int)(a)) == -2)
#endif

/* ---------------------------------------------------------------------- */

#ifndef SWIG
extern LevelValue lvNew(List dependents, Level level,
			String label, List modifees);
extern void lvDelete(LevelValue lv);
extern LevelValue lvSet(LevelValue lv, List dependents, Level level,
			String label, List regents);
extern Boolean lvCompatible(LexemGraph lg, LevelValue a, LevelValue b);
extern String lvDependent(LevelValue lv);
extern String lvRegent(LevelValue lv);
extern void lvPrint(unsigned long mode, LevelValue lv,
                    ByteVector isDeleted, int printAlternatives);
extern Connexion getCon(LevelValue a, LevelValue b);
extern Direction getDir(LevelValue x);
extern Boolean matchUnarySignature(LevelValue lv, Signature x);
extern Boolean matchBinarySignature(LevelValue lva, LevelValue lvb,
				    Signature x);
#ifdef TERNARY
extern Boolean matchTernarySignature(LevelValue lva, LevelValue lvb, LevelValue lvc,
				    Signature x);
#endif
extern Boolean lvCompare(LevelValue a, LevelValue b);
extern Boolean lvEquals(LevelValue lv1, LevelValue lv2);
extern Boolean lvCompareByLimit(LevelValue a, LevelValue b);
extern Boolean lvCompareByNaturalOrder(LevelValue a, LevelValue b);
extern LevelValue lvClone(LevelValue lv);
extern Boolean lvVectorCompatible(LevelValue lv, ByteVector taboo);
extern Boolean lvTestAllDeleted(LevelValue lv, ByteVector isDeleted);
extern int lvIndex(LevelValue lv);
extern int lvIndexAbove(LevelValue lv);
extern GraphemNode lvGetDependentGN(LevelValue lv);
extern GraphemNode lvGetRegentGN(LevelValue lv);

#endif

/* ---------------------------------------------------------------------- */
#endif /* don't insert anything after this #endif */
/** @} */
