/* 
 * Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
 *  
 * This file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. For
 * more details read COPYING in the root of this distribution.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef PREDICT_H
#define PREDICT_H

typedef struct {
  String name;  
  List input;
  String command;
} PredictorStruct;
typedef PredictorStruct *Predictor;

/* Instance of an external prediction program to feed the
   predict() CDG function.

   NAME is an (ignored) identifier in case you have
   several predictors.

   INPUT is a list of features that the predictor
   expects as input. For instance, a POS tagger would expect only the word
   form itself and have an empty list, because that is always given to the
   predictor. A supertagger might want to get the word form and the POS
   tag. A PP attacher might want the word form, the POS tag and the
   baseform. */


extern Predictor predictNew(String name, String command, List features);
extern int predictCall(Predictor p, String special, int slot, LexemGraph lg);
extern void predictAll(LexemGraph lg);
extern void predictFinalize(void);
extern void predictFree(Predictor p);

extern int predictFindPredictor(String name);

extern int predictSaveAsNormal(FILE *output, int slot, LexemGraph lg);
extern int predictSaveAsSpecial(FILE *output, String predictor, LexemGraph lg, int linecount);

extern void predictPrintLexiconInfo(LexiconItem li, FILE *input);
/* ---------------------------------------------------------------------- */
#endif /* don't insert anything after this #endif */

