/* 
 * Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
 *  
 * This file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. For
 * more details read COPYING in the root of this distribution.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

/* ---------------------------------------------------------------------------
 * @addtogroup Scache 
 * @{
 */

#ifndef SCACHE_H
#define SCACHE_H

/* ---------------------------------------------------------------------- */
#include "cdg.h"
#include "levelvalue.h"

/* -- TYPEDEFINITIONS ---------------------------------------------------- */

#ifndef SWIG

/* ---------------------------------------------------------------------------
 * This structure administers a score for each pair of LVs in a
 * constraint net.
 */
typedef struct {
  int hits;		     /**< holds the number of successful requests 
			       to the cache. */
  int count;		     /**< holds the total number of LVs registered 
			       in the cache */
  int size;		     /**< maximum used capacity */
  int capacity;		     /**< currently available capacity */
  Number *data;		     /**<  points to the underlying vector of 
			       Numbers */
} ScoreCacheStruct;
/** Pointer to ScoreCacheStruct */
typedef ScoreCacheStruct *ScoreCache;
#endif

/* -- VARIABLES --------------------------------------------------------- */

extern Boolean scUseCache;

/* -- FUNCTIONS --------------------------------------------------------- */

#ifndef SWIG
extern ScoreCache scNew(int size);
extern void scDelete(ScoreCache cache);
extern Number scGetScore(ScoreCache cache, LevelValue a, LevelValue b);
extern void scSetScore(ScoreCache cache, LevelValue a, LevelValue b, 
		       Number score);
extern void scInitialize();
#endif

/* ---------------------------------------------------------------------- */
#endif /* don't insert anything after this #endif */
/**@} */
