/*

    The BLAH library, a container library
    Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: blah@nats.informatik.uni-hamburg.de


*/

/* ----------------------------------------------------------------------
 * @defgroup Prime Primes
 * Primes is a module that makes use of Rabin's Probablistic Primetest-Algorithm
 * for generating the prime numbers equal to the Hash table entries
 * @{
 */

/* ----INCLUDES------------------------------------------------------- */
#include <math.h>
#include <time.h>
#include "blah.h"

#if !HAVE_DECL_DRAND48
  #define drand48() ((double) rand() / (double) RAND_MAX)
  #define srand48(x) (srand((x)))
#endif

/* ----------------------------------------------------------------------
 * addition in a modulo.
 *
 * @param x the first element used in addition.
 * @param y the second element used in addition
 * @param m the modulud function
 * @returns the added result.
 */
static unsigned long addMod(unsigned long x,unsigned long y,unsigned long m)
{
   return ((x + y) % m);
}

/* ----------------------------------------------------------------------
 * multiplication in a modulo.
 *
 * @param x the first element used in multiplication.
 * @param y the second element used in multiplication.
 * @param m the modulo function.
 * @returns the multiplied result.
 */
static unsigned long multMod(unsigned long x, unsigned long y, unsigned long m)
{
   unsigned long sum;

   if (x < y)
      return (multMod(y, x, m));

   for (sum = 0; y; y = y >> 1, x = addMod(x, x, m)) {
      if (y & 1)
	 sum = addMod(sum, x, m);
   }

   return sum;
}

/* ----------------------------------------------------------------------
 * exponent in a modulo.
 * 
 * @param x the base function.
 * @param y the exponent function.
 * @param m the modulo function.
 * @returns the result - x raised to the exponential y.
 */
static unsigned long powMod(unsigned long x, unsigned long y, unsigned long m)
{
   unsigned long prod;

   for (prod = 1; y; y = y >> 1, x = multMod(x, x, m)) {
      if (y & 1)
	 prod = multMod(prod, x, m);

   }
   return prod;
}

/* ----------------------------------------------------------------------
 * Rabin's probablistic primetest-algorithm
 *
 * @param number ...
 * @param times ...
 * @returns true if number is a prime by testing it a few times
 */
int primeRabin(unsigned long number, unsigned long times)
{
   unsigned long i;
   int flag;
   double x;

   for (i = 0, flag = 1; flag && i < times; i++) {
      x = floor(drand48() * (number - 1)) + 1;
      flag = (powMod((long unsigned) x, number - 1, number) == 1);
   }

   return flag;
}

/* ----------------------------------------------------------------------
 * returns the next prime after a given number.
 */
unsigned long primeNext(unsigned long number, unsigned long times)
{
   srand48(time(NULL));

   while (!primeRabin(number, times))
      number++;
   return number;
}
