/* 
 * Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
 *  
 * This file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. For
 * more details read COPYING in the root of this distribution.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Functions of the ByteVector module tested
 *
 * bvNew
 * bvAddElement
 * bvElement
 * bvRemoveElement
 * bvInsertElement
 * bvSetElement
 * bvSetElements
 * bvAndElementAt
 * bvOrElementAt
 * bvNotElementAt
 * bvSetAllElements
 * bvCapacity
 * bvSize
 * bvClone
 * bvCopy
 * bvDelete
 *
 */

#ifdef HAVE_CONFIG_H
  #include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <blah.h>
#include <assert.h>

int main(int argc, char **argv)
{
  ByteVector bv, bvAfterClone;
  int i, ind;
  unsigned char ch;
  ByteVector bvAfterCopy = bvNew(20);

  blahInitialize();

  /* ---------------------------------------- Test for bvNew,bvCapacity */

  bv = bvNew(15);
  assert(bvCapacity(bv) == 15);

  /* ---------------------------------------- Test for bvSetElements */

  bvSetElements(bv, 1, 10, 15);
  for (i = 10; i < 15; i++) {
    assert(bvElement(bv, i) == 1);
  }

  /* ---------------------------------------- Test for bvAddElement,bvSize */

  ind = bvAddElement(bv, 1);
  assert(ind == 15);
  assert(bvSize(bv) == 16);

  /* ---------------------------------------- Test for bvSetElement */

  ch = bvSetElement(bv, 1, 9);
  assert(bvElement(bv, 9) == 1);
  assert(ch == 0);

  /* -------------- Test for bvAndElementAt, bvOrElementAt, bvNotElementAt */
  bvSetElement(bv, 1, 10);
  ch = bvAndElement(bv, 3, 10);
  assert(ch == 1);
  ch = bvOrElement(bv, 2, 10);
  assert(ch == 3);
  ch = bvNotElement(bv, 10);
  assert(ch == 252);

  /* ---------------------------------------- Test for bvInsertElement */

  ch = bvInsertElement(bv, 1, 19);
  assert(bvElement(bv, 19) == 1);
  assert(ch == 0);
  assert(bvSize(bv) == 20);

  /* ---------------------------------------- Test for bvRemoveElement */

  assert(bvSize(bv) == 20);
  assert(bvElement(bv, 18) == 0);

  ch = bvRemoveElement(bv, 18);

  assert(bvElement(bv, 18) == 1);
  assert(ch == 0);
  assert(bvSize(bv) == 19);

  /* ---------------------------------------- Test for bvClone */

  bvAfterClone = bvClone(bv);
  for (i = 0; i < 19; i++) {
    assert(bvElement(bv, i) == bvElement(bvAfterClone, i));
  }
  assert(bvSize(bv) == bvSize(bvAfterClone));
  assert(bvCapacity(bv) == bvCapacity(bvAfterClone));

  /* ---------------------------------------- Test for bvSetAllElements */

  assert(bvSize(bvAfterClone) == 19);
  for (i = 0; i < 9; i++) {
    assert(bvElement(bvAfterClone, i) == 0);
  }

  bvSetAllElements(bvAfterClone, 1);

  for (i = 0; i < 19; i++) {
    assert(bvElement(bvAfterClone, i) == 1);
  }

  /* -------------------------------------- Test for bvCopy */

  bvAfterCopy = bvCopy(bvAfterCopy, bv);
  for (i = 0; i < 19; i++) {
    assert(bvElement(bv, i) == bvElement(bvAfterCopy, i));
  }
  assert(bvSize(bv) == bvSize(bvAfterCopy));
  assert(bvCapacity(bv) == bvCapacity(bvAfterCopy));

  /* -------------------------------------- ByteVectors are deleted */

  bvDelete(bv);
  bvDelete(bvAfterClone);
  bvDelete(bvAfterCopy);

  blahFinalize();
  return 0;
}
