# Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read COPYING in the root of this distribution.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
eval '(exit $?0)' && eval 'exec perl -w -S $0 ${1+"$@"}'
 & eval 'exec perl -w -S $0 $argv:q'
 if 0;

#
# Ingo Schrder
#
# modified script ga-f.pl by Horia Pop
#

require "getopts.pl";
&Getopts('hn:o:t:');

if ( $0 =~ m/.*\/([^\/]*)/ ) {
    $cmd = $1;
} else {
    $cmd = $0;
}

die "Usage: $cmd [-h] [-n <no of generation>] [-o <out file>] [-t <temp dir>]\n"
  if defined($opt_h) || defined($opt_h);

$g_max=-1;
$fitness_m_sofar=-1;
if (defined($opt_o)) {
  if ( -e $opt_o ) {
    open(O, "<$opt_o") || die "$cmd: Can't open \"$opt_o\": $!\n";
    while ($l=<O>) {
      if ($l =~ m/^(\d+)\s+[\d\.]+\s+[\d\.]+\s+[\d\.]+\s+([\d\.]+)$/ ) {
	($g, $fitness_m_sofar) = ($1, $2);
	$g_max=$g_max<$g ? $g : $g_max;
      } else {
	die "$cmd: Can't parse line in \"$opt_o\": $l";
      }
    }
    close(O);
    open(STDOUT, ">>$opt_o") || die "$cmd: Can't redirect (append) STDOUT: $!\n";
  } else {
    open(STDOUT, ">$opt_o") || die "$cmd: Can't redirect (create) STDOUT: $!\n";
  }
}

my $totalutterances = 223;
my $t = 60;

my $no_generation=defined($opt_n) ? $opt_n : 100;
my $tmpdir=defined($opt_t) ? $opt_t : "TMPDIR";
die "$cmd: <temp dir> must not contain slashes: \"$tmpdir\"\n"
  if $tmpdir =~ m+/+;
die "$cmd: Cowardly refusing to use existing <temp dir> \"$tmpdir\"\n"
  if -e $tmpdir;
mkdir($tmpdir, 0700);
chdir($tmpdir);

GENERATION:
for ($i=$g_max+1; $i<=$no_generation; $i++) {
  $fitness_m=-1;
  $f_measure_m=-1;
  $time_m=-1;

  opendir(D, "..") || die "Can't opendir ..: $!";
 ZIP:
  while ($z=readdir(D)) {
    next ZIP unless $z =~ m/^$i\.(\d+)\.zip/;
    $j=$1;
    printf STDERR "Unzipping \"%s\"... ", $z;
    system("/opt/bin/unzip ../$z > /dev/null 2>&1");
    #printf STDERR "done\n";

    $recall   = 0.0;
    $recall_c=0;
    $accuracy = 0.0;
    $accuracy_c=0;
    $time     = 0.0;
    $usertime = 0.0;
    $usertime_c=0;
    $systime  = 0.0;
    $systime_c=0;

    $jobs_c=0;

    #printf STDERR "Checking ";
    opendir(DD, ".") || die "Can't opendir .: $!";
  DDIR:
    while ($z=readdir(DD)) {
      next DDIR unless $z =~ m/d(\d\d\d\d\d)/;
      $jobs_c++;
      printf STDERR "%d ", $1;
      &SumDir($z);
    }
    close(DD);
    printf STDERR "done\n";

    printf STDERR "WARNING: #recall %d != total no of utterances %d\n",
      $recall_c, $totalutterances
	unless $recall_c == $totalutterances;
    printf STDERR "WARNING: #accuracy %d != total no of utterances %d\n",
      $accuracy_c, $totalutterances
	unless $accuracy_c == $totalutterances;

    printf STDERR "WARNING: #usertime %d != no of jobs %d\n",
      $usertime_c, $jobs_c
	unless $usertime_c == $jobs_c;
    printf STDERR "WARNING: #systime %d != no of jobs %d\n",
      $systime_c, $jobs_c
	unless $systime_c == $jobs_c;

    $recall   /= $totalutterances;
    $accuracy /= $totalutterances;

    my $time1  = $usertime + $systime;
    my $time2  = $t * $totalutterances * 1.5;
    $time      = 1 - $time1 / $time2;

    my $recall_weight = 0.5;	# recall weight inside f-measure
    my $time_weight   = 0.2;	# time weight inside fitness

    my $f_measure = 0.0;
    # this test is to avoid division-by-zero errors
    if ($recall * $accuracy != 0) {
      $f_measure = 1 / ($recall_weight / $recall + (1 - $recall_weight) / $accuracy);
    }

    my $fitness = $time_weight * $time + (1 - $time_weight) * $f_measure;

    if (0) {
      print "recall    = $recall\n";
      print "accuracy  = $accuracy\n";
      print "f_measure = $f_measure\n";
      print "usertime  = $usertime\n";
      print "systime   = $systime\n";
      print "time1     = $time1\n";
      print "time2     = $time2\n";
      print "time      = $time\n";
      print "fitness   = $fitness\n";
    } else {
      printf STDERR "%d %d %6.5f %6.5f %6.5f\n", $i, $j, $f_measure, $time, $fitness;
    }

    if ($fitness>$fitness_m) {
      $fitness_m=$fitness;
      $f_measure_m=$f_measure;
      $time_m=$time;
      if ($fitness>$fitness_m_sofar) {
	$fitness_m_sofar=$fitness;
      }
    }

    #printf STDERR "Deleting temporary files... ";
    system("rm -rf d?????");
    #printf STDERR "done\n";
  }
  close(D);

  # Jump out if nothing was found.
  last GENERATION if $f_measure_m==-1;

  printf "%d %6.5f %6.5f %6.5f %6.5f\n", $i, $f_measure_m, $time_m, $fitness_m, $fitness_m_sofar;
}

chdir("..");
rmdir($tmpdir);

exit 0;

# ------------------------------------------------------------
sub SumDir {
  local($d)=@_;
  open(FILE, "<$d/STDOUT");
  while (<FILE>) {
    $recall += $1 if m/^INFO: Recall\s+=\s+([\d\.]+)$/;
    $recall_c++ if m/^INFO: Recall\s+=\s+.*$/;

    $accuracy += $1 if m/^INFO: Accuracy\s+=\s+([\d\.]+)$/;
    $accuracy_c++ if m/^INFO: Accuracy\s+=\s+.*$/;
  }
  close(FILE);

  open(FILE, "<$d/STDERR");
  while (<FILE>) {
    if ( m/^user (.*)[,.](.*)$/ ) {
      $usertime += $1 + $2/100;
      $usertime_c++;
    }
    if ( m/^sys (.*)[,.](.*)$/ ) {
      $systime += $1 + $2/100;
      $systime_c++;
    }
  }
  close(FILE);
}


