#!/bin/sh 
# \
exec tclsh "$0" "$@" 

# Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read COPYING in the root of this distribution.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

package require tdom

# ----------------------------------------------------------------------------
# analyse a dom document and convert it to an annotation.
# ----------------------------------------------------------------------------
proc dom2anno {document} {

  set result ""

  # get the root element
  set cdgpNode [$document documentElement]
  if {$cdgpNode == ""} {
    error "ERROR: no cdgp logfile\n"
  } 

  # get the annotation node
  set annoNode [$cdgpNode descendant 1 "annotation"]
  if {$annoNode == ""} {
    error "ERROR: no annotation found in this document\n"
  }

  # get annotation and latice ids
  set id "[$annoNode @id]-xml"

  # backwards compatibility
  if {[catch { set latticeId "[$annoNode @lattice]" }]} {
    set latticeId [$cdgpNode selectNodes \
      {//parse[1]/ref[@type="Lattice"]/@name} ]
  }

  # print all infos in this dom node
  append result "'$id' : '$latticeId' <->\n"
  set isFirst 1
  foreach arcNode [$annoNode selectNodes "arc"] {

    if {$isFirst} {
      set isFirst 0
    } else {
      append result ",\n"
    }
    
    # get word info
    set from [$arcNode @from]
    set to [$arcNode @to]
    set word [$arcNode @word]
    regsub -all {'} $word {\'} word

    append result [format "%3d %3d '%s' " $from $to $word]
    foreach tagNode [$arcNode selectNodes "tag"] {
      foreach attr [$tagNode attributes] {
	append result "'$attr'/'[$tagNode getAttribute $attr]' "
      }
    }
    append result "\n"

    # get dependency info
    set levels ""
    foreach depNode [$arcNode selectNodes "dep"] {
      set level [$depNode @level]
      set label [$depNode @label]
      set modifiee [$depNode @modifiee]
      set arc($level) [list $label $modifiee]
      if {[lsearch -exact $levels $level] < 0} {
	lappend  levels $level
      }
    }


    # print dependency info
    foreach level $levels {
      set label [lindex $arc($level) 0]
      set modifiee [lindex $arc($level) 1]
      append result [format "%10s -> %-10s -> %3d\n" \
	$level "'$label'" $modifiee ]
    }

    array unset arc
  }
  append result ";\n"

  return $result
}

# ----------------------------------------------------------------------------
# convert an xml log file to a cdg input annotation
# ----------------------------------------------------------------------------
proc getAnnoFromXml {file} {

  # open the file and read in the raw data
  set fd [open "|zcat $file" r]
  set data [read $fd]
  close $fd

  # analyse the dom document
  set document [dom parse $data]
  set buffer [dom2anno $document]
  $document delete
  set document ""

  return $buffer
}

# ----------------------------------------------------------------------------
foreach {arg} $argv {
  if {[file isdirectory $arg]} {
    foreach {fileName} [glob "$arg/*.xml.gz"] {
      puts [getAnnoFromXml $fileName]
    }
  } else {
    puts [getAnnoFromXml $arg]
  }
}


