# Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read COPYING in the root of this distribution.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

## ----------------------------------------------------------------------------
## NetsearchDialog - gui for the netsearch CDG command.
##
## \author Michael Daum (see also AUTHORS and THANKS for more)
## ----------------------------------------------------------------------------
class NetsearchDialog {
   inherit ::iwidgets::Dialog

   # public methods
   public method activate {args}; ## \type TclList

   constructor {args} {}; ## \type TclList

   # private methods
   private method _ok_action {}
   
   ## list of all network ids that should be solved
   private variable _netIds ""

   ## agenda size for netsearch. defaults to 3000
   private variable _agSize 3000

   ## threshold for netsearch. defaults to 0.0
   private variable _threshold 0.0

   ## submode for netsearch. 
   ## All possible selections are alocated in the constructor being
   ## - "branchbound" (default)
   ## - "fullsearch"
   ## \todo insert all other submodes of netsearch
   private variable _submode "branchbound"
};

## ----------------------------------------------------------------------------
## constructor
## ----------------------------------------------------------------------------
body NetsearchDialog::constructor {args} {

   hide Apply
   hide Help
   buttonconfigure OK -command [code $this _ok_action]
   
   set _agSize 3000
   set _threshold 0.0
   set _submode "branchbound"

   set child [childsite]
   itk_component add selection {
      iwidgets::entryfield $child.selection \
		-labeltext "Network: "\
		-labelpos w \
		-textvariable [scope _netIds]
   } {}
   
   itk_component add _agSize {
      iwidgets::entryfield $child._agSize \
		-labeltext "AgendaSize: " \
		-labelpos w \
		-textvariable [scope _agSize] \
		-validate numeric
   } { }

   itk_component add _threshold {
      iwidgets::entryfield $child._threshold \
		-labeltext "Threshold: " \
		-labelpos w \
		-textvariable [scope _threshold] \
		-validate real
   } { }

   itk_component add method {
      iwidgets::combobox $child.method \
		-labeltext "Method: " \
		-labelpos w \
		-margin 5 \
		-labelmargin 5 \
		-textvariable [scope _submode] \
   } {
      
   }
   $itk_component(method) insert list end \
		"branchbound" \
		"fullsearch"
		
   iwidgets::Labeledwidget::alignlabels \
		$itk_component(selection) \
		$itk_component(method) \
		$itk_component(_agSize) \
		$itk_component(_threshold)

   pack $itk_component(selection) \
	$itk_component(method) \
	$itk_component(_agSize) \
	$itk_component(_threshold) \
	-side top -expand 1 -fill both -padx 5 -pady 5

   eval itk_initialize $args
}

## ----------------------------------------------------------------------------
## activate the netsearch dialog.
## \param args an optional list of network ids that should be preselected.
## ----------------------------------------------------------------------------
body NetsearchDialog::activate {args} {

   eval set _netIds $args

   center .
   Shell::activate 
}

## ----------------------------------------------------------------------------
## slot for the ok button action.
## This function is bound to the "ok" button of the gui, which then deactivates
## the dialog and calls ::cmd::netsearch().
## ----------------------------------------------------------------------------
body NetsearchDialog::_ok_action {} {
   deactivate
   foreach id $_netIds {
     ::cmd::Netsearch $id $_submode $_agSize $_threshold
   }
}
