#!/bin/sh
#\
exec wish "$0" "$@" 

# ----------------------------------------------------------------------------
# Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read COPYING in the root of this distribution.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# ----------------------------------------------------------------------------

## ----------------------------------------------------------------------------
## \mainpage The XCDG Reference Manual
## \image html xcdg.gif
## \author Michael Daum, Kilian A. Foth, Dietmar Fnning
##
## \section Overview Overview
## This is the programmers reference manual of XCDG, that is a documentation
## of the implementation. For a detailed usage overview please consult the 
## Xcdg User Manual.
##
## \section Copyright Copyright
## Copyright (C) 1997-2004 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
##
## \verbatim
## XCDG is licensed under the GPL.
##  
## This application is free software; as a special exception the author gives
## unlimited permission to copy and/or distribute it, with or without 
## modifications, as long as this notice is preserved.
## 
## This program is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
## implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
## \endverbatim
##
## ----------------------------------------------------------------------------

## \comment

## switch this to 1 to enable the experimental cycle drawing code
set EXPERIMENTAL_CYCLE_DRAWING 0

if {![info exists env(XCDG_LIB)]} {
   set env(XCDG_LIB) {.}
}

if {![file exists $env(XCDG_LIB)]} {
   puts "ERROR: can't locate xcdg-directory, please set XCDG_LIB"
   exit
}

set env(XCDG_LIB) [glob $env(XCDG_LIB)]
lappend auto_path $env(XCDG_LIB)

# load some libraries:

# XML parsing for tcl
package require tdom

# Itcl -- object-oriented extensions to Tcl
package require Iwidgets

# required so we can just write "class foo {...}"
namespace import -force itcl::*
tk_focusFollowsMouse

# otherwise we would have to write "itcl::class foo {...}"

# a better table widget than the standard one
package require Tktable

# libcdg calls
load $env(CDG_BINDINGS)

## ----------------------------------------------------------------------------
## process the command line options given to xcdg etc.
## ----------------------------------------------------------------------------
proc read_argv {} {
  global argv noinit execFiles quiet grammarpath usermode showmain cdgXCDG

   # standard values 
   set noinit 0
   set execFiles ""
   set quiet false
   set grammarpath ""
   set usermode ""
   set showmain true
   
   set cdgXCDG 1

   # parse parameters - set values
   foreach {flag value} $argv {
      set done false
      while { !$done } {
      
        switch -- $flag {
	   -i {
	      if {"$value" == "noinit" || "$value" == "none" || "$value" == "NULL"} {
	        set noinit 1
	      } else {
	        lappend execFiles $value
	      }
	      set done true
	   }
	   -grammarpath {
	      set grammarpath $value
	      set done true
	   }
	   -noinit {
	      set noinit 1
	   }
	   -user {
	      puts "starting up in user mode !"
	      set usermode "user"
	   }
	   -nosplash {
	      set quiet true
	   }
	   -nomain {
	      set showmain false
	   }
	   default {
	      puts stderr "unknown option `$flag'"
	      puts stderr "usage: xcdg \[-i <scriptfile\]* \n\
			  \t\[-grammarpath <path-to-grammars>\] \n\
			  \t\[-noinit\]"
	      exit 1
	   }
	}
	    
	if { !$done } {
		if { $value == "" } {
			set done true
		} else {
			set flag $value
			set value ""
		}
	}
      }
   }
}

read_argv

# display the startup-screen
wm withdraw .
if { $quiet } {
	set st [StartUpQuiet .startup]
} else {
	set st [StartUp .startup -title "Xcdg - Splash"] 
}

# this kind of statement updates the startup-screen's progress bar
$st step 2

## ----------------------------------------------------------------------------
## called by the flush-hook
## ----------------------------------------------------------------------------
proc cdgFlush {} {
   global cdgFlushClock

   set now [clock clicks]
   set duration [expr abs($now - $cdgFlushClock)]
   if {$duration > 150000} {
      update 
      set cdgFlushClock $now
   }
}
## ----------------------------------------------------------------------------
## changes a string from latin-1 to UTF if the Tcl Version is 8.5
## ----------------------------------------------------------------------------
proc fixencoding {s} {
  if {[info tclversion] == "8.5"} {
    return [encoding convertfrom "iso8859-15" $s]
  } else {
    return $s
  }
}
proc unfixencoding {s} {
  if {[info tclversion] == "8.5"} {
    return [encoding convertto "iso8859-15" $s]
  } else {
    return $s
  }
}


# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------
# initial steps

. configure -highlightthickness 0
if {[catch { option readfile $env(XCDG_LIB)/Cdgrc userDefault } errMsg]} {
   puts "error in reading Cdgrc: "
   puts $errMsg
   exit 1
}



# set the hooks
setHookCmd flush "cdgFlush"
setHookCmd printf "::cmd::Puts -nonewline"
setHookCmd gets ".cdgmain shell fgets"
setHookCmd progress ".cdgmain help showstr"
setHookCmd partialresult ".cdgmain parses handlePartialResult"
setHookCmd "IC interaction" ".cdgmain parses handleICinteraction"
setHookCmd reset ".cdgmain parses reset"

set cdgFlushClock 0
commandEval "hook flush on"
commandEval "hook printf on"

# build the application
CdgMain .cdgmain
if { $grammarpath != "" } {
	.cdgmain configure -grammarpath $grammarpath

}
if { $usermode != "" } {
    .cdgmain mode $usermode
}
pack .cdgmain -fill both -expand 1
destroy $st
scan [wm maxsize .] "%d %d" maxx maxy
set maxx [expr $maxx - 100]
wm geometry . ${maxx}x${maxy}+100+0
wm minsize . 800 600
wm title . "Xcdg - Main"
update idletask

# source bugfixes: don't rely on autoload
source $env(XCDG_LIB)/bugfix.tcl

# print some ugly messages
set xcdg_version {$Id: xcdg.tcl,v 1.109 2009/12/16 16:38:12 foth Exp $}

::cmd::Puts "
INFO: \[X\]CDG parser v[lindex $xcdg_version 2], [lindex $xcdg_version 3]
      For more information see
          http://nats-www.informatik.uni-hamburg.de/view/Papa
      The CDG Team can be contacted at
          cdg@nats.informatik.uni-hamburg.de
"
# user mode ?!?
if {[.cdgmain mode] != "user"} {
  if { $showmain } { wm deiconify . }
} else {
  DemoWindow .demowindow
}

# execute actions specified in .xcdgrc
if {$noinit == 0 && "$execFiles" == ""} {
  if {[file exists ./.xcdgrc]} {
   .cdgmain shell safeSource ./.xcdgrc 
  } else {
    if {[file exists ~/.xcdgrc]} {
      .cdgmain shell safeSource ~/.xcdgrc 
    }
  }
}

   # reset shell
   .cdgmain shell resetCmd

   # execute collected files
   foreach file $execFiles {
      if {$file == "stdin"} {
	 while {![eof stdin]} {
	    .cdgmain shell safeEval [read stdin]
	 }
      } else {
	 if {[file exists $file]} {
	    .cdgmain shell safeSource $file
	 } else {
	    ::cmd::Puts "ERROR: can't find file \"$file\""
	 }
      }
   }


## \endcomment

