# Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read COPYING in the root of this distribution.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

## ----------------------------------------------------------------------------
## YadaGlsStats - extra statistics for gls.
## \ingroup YadaRunnableDocuments
##
## \author Michael Daum
##
## ----------------------------------------------------------------------------
class YadaGlsStats {
  inherit YadaRunnableDocument

  # variables ----------------------------------------------------------------
  private variable _totalTime 0
  private variable _totalRecall 0.0
  private variable _minUtility 0
  private variable _maxCosts 0
  private variable _maxDetour 0
  private variable _maxTime 0
  private variable _maxCycles 0
  private variable _typeOfColumn ;## \type TclArray
  private variable _lexicalHitsOf ;## \type TclArray
  private variable _triesOf ;## \type TclArray

  # methods ------------------------------------------------------------------
  public method activationHandle {}
  public method clear {}
  public method sortRows {colNo args}; ## \type TclNumber, TclList

  constructor {args} {}; ## \type TclList

  private method _analyseDocument {document experiment}; ## \type DomDocument, YadaExperiment
  private method _computeStatistics {}
  private method _showStatistics {}
  private method _getTypeOfColumn {colNo} ;## \type TclNumber
  private method _judgeRow {row}; ## \type TclNumber
  private method _resetDataOf {wordgraphs};## \type TclList
};

## ----------------------------------------------------------------------------
## constructor
## ----------------------------------------------------------------------------
body YadaGlsStats::constructor {args} {

  # reconfigure table
  $itk_component(table) configure \
    -rows 300 -cols 17 \
    -cellanchor e \
    -rowtagcommand [code $this _judgeRow]

  # set types of columns
  array set _typeOfColumn {
    0 "dictionary"
    1 "integer"
    2 "real"
    3 "integer"
    4 "real"
    5 "integer"
    6 "integer"
    7 "real"
    8 "real"
    9 "real"
    10 "real"
    11 "real"
    12 "real"
    13 "integer"
    14 "integer"
    15 "integer"
  }


  itk_component add experimentBoxLabel {
    label $itk_component(heading).experimentBoxLabel \
	-text "Experiment:" \
	-anchor w
  }  {}

  # experiment box
  itk_component add experimentBox {
    iwidgets::combobox $itk_component(heading).experimentBox \
	-completion false \
	-borderwidth 2 \
	-margin 4 \
	-selectioncommand [code $this selectExperiment]
  }  {}

  # statistics
  itk_component add totalTimeLabel {
    label $itk_component(heading).totalTimeLabel \
      -text "Time:" \
      -anchor w 
  } {}

  itk_component add totalTime {
    entry $itk_component(heading).totalTime \
      -borderwidth 2 \
      -state disabled \
      -width 20
  } {}

  itk_component add minUtilityLabel {
    label $itk_component(heading).minUtilityLabel \
      -text "Min.Utility:" \
      -anchor w 
  } {}

  itk_component add minUtility {
    entry $itk_component(heading).minUtility \
      -borderwidth 2 \
      -state disabled \
      -width 20
  } {}

  itk_component add maxCostsLabel {
    label $itk_component(heading).maxCostsLabel \
      -text "Max.Costs:" \
      -anchor w 
  } {}

  itk_component add maxCosts {
    entry $itk_component(heading).maxCosts \
      -borderwidth 2 \
      -state disabled \
      -width 20
  } {}

  itk_component add maxDetourLabel {
    label $itk_component(heading).maxDetourLabel \
      -text "Max.Detour:" \
      -anchor w 
  } {}

  itk_component add maxDetour {
    entry $itk_component(heading).maxDetour \
      -borderwidth 2 \
      -state disabled \
      -width 20
  } {}

  itk_component add maxTimeLabel {
    label $itk_component(heading).maxTimeLabel \
      -text "Max.Time:" \
      -anchor w 
  } {}

  itk_component add maxTime {
    entry $itk_component(heading).maxTime \
      -borderwidth 2 \
      -state disabled \
      -width 20
  } {}

  itk_component add maxCyclesLabel {
    label $itk_component(heading).maxCyclesLabel \
      -text "Max.Cycles:" \
      -anchor w 
  } {}

  itk_component add maxCycles {
    entry $itk_component(heading).maxCycles \
      -borderwidth 2 \
      -state disabled \
      -width 20
  } {}

  itk_component add totalRecallLabel {
    label $itk_component(heading).totalRecallLabel \
      -text "Lex.Recall:" \
      -anchor w 
  } {}

  itk_component add totalRecall {
    entry $itk_component(heading).totalRecall \
      -borderwidth 2 \
      -state disabled \
      -width 20
  } {}

 # bindings
  set entry [$itk_component(experimentBox) component entry]
  bind $entry <Return> [code $this selectExperiment]
  bind $entry <Up> +[code $this selectExperiment]
  bind $entry <Down> +[code $this selectExperiment]

  # packing
  grid columnconfigure $itk_component(heading) 1 -weight 1
  grid columnconfigure $itk_component(heading) 2 -pad 5
  grid columnconfigure $itk_component(heading) 3 -weight 1

  grid $itk_component(experimentBoxLabel)   -row 0 -column 0 -sticky ew -pady 2
  grid $itk_component(experimentBox)        -row 0 -column 1 -sticky ew -pady 2
  grid $itk_component(totalRecallLabel) -row 0 -column 2 -sticky ew -pady 2
  grid $itk_component(totalRecall)      -row 0 -column 3 -sticky ew -pady 2
  grid $itk_component(totalTimeLabel)   -row 1 -column 0 -sticky ew -pady 2
  grid $itk_component(totalTime)        -row 1 -column 1 -sticky ew -pady 2
  grid $itk_component(minUtilityLabel)  -row 1 -column 2 -sticky ew -pady 2
  grid $itk_component(minUtility)       -row 1 -column 3 -sticky ew -pady 2
  grid $itk_component(maxCostsLabel)    -row 3 -column 0 -sticky ew -pady 2
  grid $itk_component(maxCosts)         -row 3 -column 1 -sticky ew -pady 2
  grid $itk_component(maxDetourLabel)   -row 3 -column 2 -sticky ew -pady 2
  grid $itk_component(maxDetour)        -row 3 -column 3 -sticky ew -pady 2
  grid $itk_component(maxTimeLabel)     -row 4 -column 0 -sticky ew -pady 2
  grid $itk_component(maxTime)          -row 4 -column 1 -sticky ew -pady 2
  grid $itk_component(maxCyclesLabel)   -row 4 -column 2 -sticky ew -pady 2
  grid $itk_component(maxCycles)        -row 4 -column 3 -sticky ew -pady 2


  eval itk_initialize $args
  .main registerDocument $this
}

## ----------------------------------------------------------------------------
## clear
## ----------------------------------------------------------------------------
body YadaGlsStats::clear {} {
  chain ;# call YadaRunnableDocument::clear

  # reset the table 
  set table $itk_component(table)
  $table setCell \
       	0,0 "wordgraph" \
	0,1 "values" \
	0,2 "score" \
	0,3 "errors" \
	0,4 "recall" \
	0,5 "cycles" \
	0,6 "time" \
	0,7 "sol.time" \
	0,8 "soft.time" \
       	0,9 "min.util" \
       	0,10 "needed util" \
	0,11 "max.costs" \
	0,12 "needed costs" \
       	0,13 "max.detour" \
       	0,14 "needed detour" \
	0,15 "blind alleys" \

  $table tag config default -anchor e -relief flat -bd 2
  $table tag config error -bg red3 -fg white -anchor e -relief flat -bd 2
  $table tag config colored -bg gray80 -anchor e

  $table tag config sel -fg "" -bg "" -relief raised -bd 1
  $table tag config active -anchor w -fg black -bg gray -relief sunk -bd 1
  $table tag config title -font {-adobe-helvetica-medium-r-normal--12-*-*-*-*-*-*-*} -bg #a3a3a3 -fg white
  $table tag config runner -fg red3 -bg #a3a3a3
  $table width 0 15 1 5 2 10 3 5 4 6 5 7
  $table tag raise runner title

  # reset scores of selected wordgraphs
  #_resetDataOf [_getActiveWordgraphs]
  
  _computeStatistics
}

## ----------------------------------------------------------------------------
## _analyseDocument
## ----------------------------------------------------------------------------
body YadaGlsStats::_analyseDocument {document experiment} {

  # default analysis
  YadaRunnableDocument::_analyseDocument $document $experiment

  # the wordgraph is needed at least
  if {$_analyseResult(wordgraph) == ""} {
    print "ERROR: no information available in this run\n"
    return
  }

  #
  # special analysis
  #
  set _analyseResult(minUtility) 0
  set _analyseResult(minUtilityNeeded) 0
  set _analyseResult(maxCosts) 0
  set _analyseResult(maxCostsNeeded) 0
  set _analyseResult(maxDetour) 0
  set _analyseResult(maxDetourNeeded) 0
  set _analyseResult(blindAlleys) 0

  
  # get the root node
  set cdgpNode [$document documentElement]

  # parse the statistics 
  foreach aspect {steps minUtility 
    minUtilityNeeded maxDetour maxDetourNeeded maxCosts maxCostsNeeded blindAlleys } {
    set _analyseResult($aspect) [[$cdgpNode selectNodes \
      descendant::statistics/number\[@name="$aspect"\]] @value ]
    if {$_analyseResult($aspect) == ""} {
      set _analyseResult($aspect) 0
    }
  }

  set _lexicalHitsOf($_analyseResult(wordgraph)) $_analyseResult(lexicalHits)
  set _triesOf($_analyseResult(wordgraph)) $_analyseResult(tries)
  if {$_analyseResult(tries) > 0} {
    set lexicalRecall [expr $_analyseResult(lexicalHits) * 100.0 / $_analyseResult(tries) ]
  } else {
    set lexicalRecall 0
  }


  # add the collected data to the table
  _insertData $_analyseResult(wordgraph) \
	      $_analyseResult(noValues) \
	      [format "%4.3e" $_analyseResult(score)] \
	      [expr $_analyseResult(tries) - $_analyseResult(lexicalHits)] \
	      [format "%.2f" $lexicalRecall] \
	      $_analyseResult(steps) \
	      $_analyseResult(totalTime) \
	      $_analyseResult(solutionTime) \
	      $_analyseResult(softTime) \
	      [format "%4.3e" $_analyseResult(minUtility)] \
	      [format "%4.3e" $_analyseResult(minUtilityNeeded)] \
	      [format "%.2f" $_analyseResult(maxCosts)] \
	      [format "%.2f" $_analyseResult(maxCostsNeeded)] \
	      $_analyseResult(maxDetour) \
	      $_analyseResult(maxDetourNeeded) \
	      $_analyseResult(blindAlleys)
}

## ----------------------------------------------------------------------------
## activationHandle
## ----------------------------------------------------------------------------
body YadaGlsStats::activationHandle {} {
  chain ;# call YadaRunnableDocument::activationHandle

  foreach experiment [.main getExperiments] {
    set experimentName [$experiment cget -name]
    if {$experimentName == "<none>" ||
	([$experiment cget -isActive] && [$experiment cget -type] == "gls")} {
      lappend experimentNames $experimentName
    }
  }

  # get the previous selection
  set box $itk_component(experimentBox)
  set experimentName [$box get]

  # default experiments to choose
  if {$experimentName == ""} {
    set experimentName "<none>"
  }

  set listOfEntries [[$box component list] get 0 end]
  if {$listOfEntries != $experimentNames} {
    $box clear list 
    eval $box insert list end $experimentNames
    selectExperiment $experimentName
  }
}

## ----------------------------------------------------------------------------
## _computeStatistics
## ----------------------------------------------------------------------------
body YadaGlsStats::_computeStatistics {} {
  chain

  set table $itk_component(table)
  set rmax [$table cget -rows]

  set _totalTime 0
  set _totatLexicalRecall 0.0
  set _minUtility 999999999
  set _maxCosts 0
  set _maxDetour 0
  set _maxTime 0
  set _maxCycles 0

  set totalTries 0
  set totalLexicalHits 0

  for {set row 1} {$row < $rmax} {incr row} {

    set dataRow [$table getCell $row,0 $row,14]
    set wordgraph               [lindex $dataRow 0]
#    set noValues                [lindex $dataRow 1]
#    set score                   [lindex $dataRow 2]
    set errors			[lindex $dataRow 3]
    set cycles			[lindex $dataRow 5]
    set time			[lindex $dataRow 6]
    set minUtilityNeeded	[lindex $dataRow 7]
    set maxCostsNeeded		[lindex $dataRow 12]
    set maxDetourNeeded		[lindex $dataRow 14]

    if {$errors == ""} {
      continue
    }

    incr _totalTime $time
    if {$_minUtility > $minUtilityNeeded} {
      set _minUtility $minUtilityNeeded
    }
    if {$_maxCosts < $maxCostsNeeded} {
      set _maxCosts $maxCostsNeeded
    }
    if {$_maxDetour < $maxDetourNeeded} {
      set _maxDetour $maxDetourNeeded
    }
    if {$_maxTime < $time} {
      set _maxTime $time
    }
    if {$_maxCycles < $cycles} {
      set _maxCycles $cycles
    }

    incr totalTries $_triesOf($wordgraph)
    incr totalLexicalHits $_lexicalHitsOf($wordgraph)
  }

  if {$totalTries > 0} {
    set _totalRecall [expr $totalLexicalHits * 100.0 / $totalTries ]
  }

  _showStatistics
}

## ----------------------------------------------------------------------------
## _showStatistics
## ----------------------------------------------------------------------------
body YadaGlsStats::_showStatistics {} {
  $itk_component(totalTime) configure -state normal
  $itk_component(minUtility) configure -state normal
  $itk_component(maxCosts) configure -state normal
  $itk_component(maxDetour) configure -state normal
  $itk_component(maxTime) configure -state normal
  $itk_component(maxCycles) configure -state normal
  $itk_component(totalRecall) configure -state normal

  $itk_component(totalTime) delete 0 end
  $itk_component(minUtility) delete 0 end
  $itk_component(maxCosts) delete 0 end
  $itk_component(maxDetour) delete 0 end
  $itk_component(maxTime) delete 0 end
  $itk_component(maxCycles) delete 0 end
  $itk_component(totalRecall) delete 0 end

  $itk_component(totalTime) insert 0 "[timeToString $_totalTime]"
  if {$_minUtility == 999999999} {
    $itk_component(minUtility) insert 0 "<min>"
  } else {
    $itk_component(minUtility) insert 0 [format "%4.3e" $_minUtility]
  }
  $itk_component(maxCosts) insert 0 [format "%.2f" $_maxCosts]
  $itk_component(maxDetour) insert 0 $_maxDetour
  $itk_component(maxTime) insert 0 $_maxTime
  $itk_component(maxCycles) insert 0 $_maxCycles
  $itk_component(totalRecall) insert 0 [format "%.4f" $_totalRecall]

  $itk_component(totalTime) configure -state disabled
  $itk_component(minUtility) configure -state disabled
  $itk_component(maxCosts) configure -state disabled
  $itk_component(maxDetour) configure -state disabled
  $itk_component(maxTime) configure -state disabled
  $itk_component(maxCycles) configure -state disabled
  $itk_component(totalRecall) configure -state disabled
}

## ----------------------------------------------------------------------------
## _getTypeOfColumn
## ----------------------------------------------------------------------------
body YadaGlsStats::_getTypeOfColumn {colNo} {
  return $_typeOfColumn($colNo)
}

## ----------------------------------------------------------------------------
## sortRows
## ----------------------------------------------------------------------------
body YadaGlsStats::sortRows {colNo args} {
  eval chain $colNo $args
  _computeStatistics
}

## ----------------------------------------------------------------------------
## _judgeRow
## ----------------------------------------------------------------------------
body YadaGlsStats::_judgeRow {row} {
  set recall [$itk_component(table) getCell $row,4]
  if {$recall != "" && $recall < 100.0} {
    return "error"
  } else {
    if {[expr $row % 2]} {
      return "default"
    } else {
      return "colored"
    }
  }
}

## ----------------------------------------------------------------------------
## _resetDataOf
## ----------------------------------------------------------------------------
body YadaGlsStats::_resetDataOf {wordgraphs} {
  chain $wordgraphs
  foreach wordgraph $wordgraphs {
    set _triesOf($wordgraph) 0
    set _lexicalHitsOf($wordgraph) 0
  }
}

